/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_AUTHINFO_H
#define __EPP_AUTHINFO_H

/** @file epp_AuthInfo.h
 *  @brief Definition of epp_AuthInfo and epp_AuthInfo_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"

namespace eppobject { namespace epp {
   
   /// Class containing authorization information associated with an object   
   class epp_AuthInfo {
      
    public:
      epp_AuthInfoType_ref m_type; /**< Authorization information type */
      epp_roid_ref m_roid; /**< Repository object ID of associated contact object */
      epp_string_ref m_value; /**< Authorization string/passphrase */
      
      /// Null constructor
      epp_AuthInfo() {};
      
      /// Destructor
      virtual ~epp_AuthInfo() {};
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_AuthInfo(const epp_AuthInfoType & _m_type,
		   const epp_roid & _m_roid,
		   const epp_string & _m_value)
	{
	   m_type.ref(new epp_AuthInfoType(_m_type));
	   m_roid.ref(new epp_roid(_m_roid));
	   m_value.ref(new epp_string(_m_value));
	};
      
      /// Constructor, using only references
      epp_AuthInfo(const epp_AuthInfoType_ref & _m_type,
		   const epp_roid_ref & _m_roid,
		   const epp_string_ref & _m_value)
	{
	   m_type = _m_type;
	   m_roid = _m_roid;
	   m_value = _m_value;
	};
   };
   
   /// Typedef for the ref to the epp_AuthInfo class
   typedef refcnt_ptr<epp_AuthInfo> epp_AuthInfo_ref;
   
}}

#endif
