/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_CONTACTCREATERSP_H
#define __EPP_CONTACTCREATERSP_H

/** @file epp_ContactCreateRsp.h
 *  @brief Definition of epp_ContactCreateRsp and epp_ContactCreateRsp_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_contactdata.h"
#include "epp_GenericRsp.h"

namespace eppobject { namespace contact {
   
   /// Class containing elements used in contact create responses
   /// @see epp_GenericRsp for additional information
   class epp_ContactCreateRsp : public epp::epp_GenericRsp {
      
    public:
      eppobject::epp::epp_string_ref m_id; /**< Server-unique identifier for the created contact */
      eppobject::epp::epp_datetime_ref m_creation_date; /**< Date and time of contact object creation */
      
      /// Null constructor
      epp_ContactCreateRsp() {};
      
      /// Destructor
      virtual ~epp_ContactCreateRsp() {};

      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_ContactCreateRsp(const eppobject::epp::epp_Response_ref & _m_rsp,
			   const eppobject::epp::epp_string & _m_id,
			   const eppobject::epp::epp_datetime & _m_creation_date)
	: epp::epp_GenericRsp(_m_rsp)
	{
	   m_id.ref(new eppobject::epp::epp_string(_m_id));
	   m_creation_date.ref(new eppobject::epp::epp_datetime(_m_creation_date));
	};
      
      /// Constructor, using only references
      epp_ContactCreateRsp(const eppobject::epp::epp_Response_ref & _m_rsp,
			   const eppobject::epp::epp_string_ref & _m_id,
			   const eppobject::epp::epp_datetime_ref & _m_creation_date)
	: epp::epp_GenericRsp(_m_rsp)
	{
	   m_id = _m_id;
	   m_creation_date = _m_creation_date;
	};
   };
   
   /// Typedef for the ref to the epp_ContactCreateRsp class
   typedef refcnt_ptr<epp_ContactCreateRsp> epp_ContactCreateRsp_ref;
   
}}

#endif
