/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_CONTACTNAMEADDRESS_H
#define __EPP_CONTACTNAMEADDRESS_H

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/data/epp_ContactNameAddress.h,v 1.12 2003/07/18 20:21:19 tubadanm Exp $ */

/** @file epp_ContactNameAddress.h
 *  @brief Definition of epp_ContactNameAddress and epp_ContactNameAddress_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_contactdata.h"
#include "epp_ContactAddress.h"

namespace eppobject { namespace contact {
   
   /// Class containing elements used to describe the name and address of a contact
   class epp_ContactNameAddress {
      
    public:
      eppobject::epp::epp_string_ref m_name; /**< Name of the contact */
      eppobject::epp::epp_string_ref m_org; /**< Organization associated with the contact */
      epp_ContactAddress_ref m_address; /**< Address associated with the contact */
      epp_ContactPostalInfoType_ref m_type; /** The type of the address: INTernationalize or LOCalized */
      
      /// Null constructor
      epp_ContactNameAddress() {};
      
      /// Destructor
      virtual ~epp_ContactNameAddress() {};
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_ContactNameAddress(const eppobject::epp::epp_string & _m_name,
			     const eppobject::epp::epp_string & _m_org,
			     const epp_ContactAddress_ref & _m_address,
                             const epp_ContactPostalInfoType & _m_type = INT)
	{
	   m_name.ref(new eppobject::epp::epp_string(_m_name));
	   m_org.ref(new eppobject::epp::epp_string(_m_org));
	   m_address = _m_address;
           m_type.ref(new epp_ContactPostalInfoType(_m_type));
	};
      
      /// Constructor, using only references
      epp_ContactNameAddress(const eppobject::epp::epp_string_ref & _m_name,
			     const eppobject::epp::epp_string_ref & _m_org,
			     const epp_ContactAddress_ref & _m_address,
                             const epp_ContactPostalInfoType_ref & _m_type = NULL)
	{
	   m_name = _m_name;
	   m_org = _m_org;
	   m_address = _m_address;
           m_type = _m_type;
	};
   };

   /// Typedef for the ref to the epp_ContactNameAddress class
   typedef refcnt_ptr<epp_ContactNameAddress> epp_ContactNameAddress_ref;
   typedef std::vector<epp_ContactNameAddress> epp_ContactNameAddress_seq;
   typedef refcnt_ptr<epp_ContactNameAddress_seq> epp_ContactNameAddress_seq_ref;
   
   std::ostream &operator<<(std::ostream &stream, const epp_ContactNameAddress_ref &cna);
   std::ostream &operator<<(std::ostream &stream, const epp_ContactNameAddress &cna);
   
}}

#endif
