/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_CONTACTPANDATA_H
#define __EPP_CONTACTPANDATA_H

/** @file epp_ContactPanData.h
 *  @brief Definition of epp_ContactPanData and epp_ContactPanData_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"
#include "epp_PollResData.h"
#include "epp_TransID.h"

namespace eppobject { namespace contact {

   /// Class containing elements describing a contact transfer.
   /// @see epp_PollResData for additional information
   class epp_ContactPanData : public epp::epp_PollResData {
      
    public:
      epp::epp_string_ref m_id; /**< The ID of the contact object */
      epp::epp_boolean_ref m_result; /**< */
      epp::epp_TransID_ref m_trid; /**< The TRID from the request which submitted this pending action */
      epp::epp_datetime_ref m_date; /**< Date and time the pending action was completed (to success or failure) */
      
      /// Null constructor, setting #m_type
      epp_ContactPanData()
	{
	   m_type = "contact:panData";
	};
      
      /// Destructor
      virtual ~epp_ContactPanData() {}
      
      /// Constructor, using simple types when it can, setting #m_type
      /// @note Cannot set simple types to NULL
      epp_ContactPanData(const epp::epp_string & _m_id,
		         const epp::epp_boolean & _m_result,
		         const epp::epp_TransID & _m_trid,
		         const epp::epp_datetime & _m_date)
	{
	   m_type = "contact:panData";
	   
	   m_id.ref(new epp::epp_string(_m_id));
	   m_result.ref(new epp::epp_boolean(_m_result));
	   m_trid.ref(new epp::epp_TransID(_m_trid));
	   m_date.ref(new epp::epp_datetime(_m_date));
	};
      
      /// Constructor, using only references, setting #m_type
      epp_ContactPanData(const epp::epp_string_ref & _m_id,
		         const epp::epp_boolean_ref & _m_result,
		         const epp::epp_TransID_ref & _m_trid,
		         const epp::epp_datetime_ref & _m_date)
	{
	   m_type = "contact:panData";

	   m_id = _m_id;
	   m_result = _m_result;
	   m_trid = _m_trid;
	   m_date = _m_date;
	};
      
      /// Returns the type of object
      /// @return Type of epp_PollResData object
      epp::epp_string getType()
	{
	   return m_type;
	};
      
      /** Parses a panData dom_ptr, fills in the public attributes of this class,
       *  and returns a ref to an #epp_PollResData object that is really an
       *  epp_ContactPanData object
       * @param trnNode The panData dom_ptr node
       * @return The corresponding epp_ContactPanData object
       */
      epp::epp_PollResData_ref createPollResData(const domtools::dom_ptr & trnNode);

   };
   
   /// Typedef for the ref to the epp_ContactPanData class
   typedef refcnt_ptr<epp_ContactPanData> epp_ContactPanData_ref;
   
   /// Creates a pointer to an epp_ContactPanData-object and returns it
   /// as an epp_PollResData pointer
   /// @return Pointer to an epp_ContactPanData object
   static epp::epp_PollResData * createContactPanResData() {
      return new epp_ContactPanData();
   }
   
}}

#endif
