/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/


/** @file epp_ContactTrnData.cc
 *  @brief Additional definitions for epp_ContactTrnData
 *  @author Asbjorn Steira Mikkelsen
 *  @see epp_ContactTrnData.h
 */

#include <domtools/dom_wrapper.h>
#include "epp_ContactTrnData.h"
#include "epp_contactXMLbase.h"

using namespace std;
using namespace eppobject::contact;
using namespace eppobject::epp;
using namespace domtools;

epp_PollResData_ref epp_ContactTrnData::createPollResData(const dom_ptr & trnNode)
{
   epp_ContactTrnData_ref cont(new epp_ContactTrnData());
   
   if(!trnNode->isNull()) {
      
      dom_ptr CTChildNode = dom_ptr(trnNode->getFirstChildElement());
      
      string elemPrefix = trnNode->getPrefix();
      
      while(!CTChildNode->isNull()) {
	 string nodeName = CTChildNode->getNodeName();
	 
	 if(nodeName == string(elemPrefix + ":id")) {
	    cont->m_id.ref(new epp_string(CTChildNode->getNodeData()));
//	    cout << "id: " << *cont->m_id << endl;   
	 } // if "id"
	 
	 else if(nodeName == string(elemPrefix + ":trStatus")) {
	    string trStatus = CTChildNode->getNodeData();
	    cont->m_transfer_status.ref(new epp_TransferStatusType(returnTransferStatusType(trStatus)));
//	    cout << "trStatus: " << trStatus << endl;
	 } // if "trStatus"
	 
	 else if(nodeName == string(elemPrefix + ":reID")) {
	    cont->m_request_client_id.ref(new epp_string(CTChildNode->getNodeData()));
//	    cout << "reID: " << *cont->m_request_client_id << endl;
	 } // if "reID"
	 
	 else if(nodeName == string(elemPrefix + ":reDate")) {
	    cont->m_request_date.ref(new epp_string(CTChildNode->getNodeData()));
//	    cout << "reDate: " << *cont->m_request_date << endl;
	 } // if "reDate"
	 
	 else if(nodeName == string(elemPrefix + ":acID")) {
	    cont->m_action_client_id.ref(new epp_string(CTChildNode->getNodeData()));
//	    cout << "acID: " << *cont->m_action_client_id << endl;        
	 } // if "acID"
	 
	 else if(nodeName == string(elemPrefix + ":acDate")) {
	    cont->m_action_date.ref(new epp_string(CTChildNode->getNodeData()));
//	    cout << "acDate: " << *cont->m_action_date << endl;
	 } // if "acDate"
	 
	 CTChildNode = dom_ptr(CTChildNode->getNextSiblingElement());
      } // while "CTChildNode"
      
   } // if "trnNode"
   
   return cont;  
}
