/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_DOMAININFORSP_H
#define __EPP_DOMAININFORSP_H

/** @file epp_DomainInfoRsp.h
 *  @brief Definition of epp_DomainInfoRsp and epp_DomainInfoRsp_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_domaindata.h"
#include "epp_GenericRsp.h"
#include "epp_DomainStatus.h"
#include "epp_DomainContact.h"
#include "epp_AuthInfo.h"

namespace eppobject { namespace domain {
   
   /// Class containing elements used in domain info responses
   /// @see epp_GenericRsp for additional information
   class epp_DomainInfoRsp : public epp::epp_GenericRsp {
      
    public:
      eppobject::epp::epp_string_ref m_name; /**< Fully qualified name of the domain object */
      eppobject::epp::epp_roid_ref m_roid; /**< Repository Object IDentifier assigned to the domain */
      epp_domain_status_seq_ref m_status; /**< Status of the domain object */
      eppobject::epp::epp_string_ref m_registrant; /**< ID for the contact object associated with the
						    * domain object as the object registrant */
      epp_domain_contact_seq_ref m_contacts; /**< IDs for other contact objects associated with the domain object */
      eppobject::epp::epp_string_seq_ref m_name_servers; /**< Fully qualified names of the delegated host objects
							  * (name servers) associated with the domain object */
      eppobject::epp::epp_string_seq_ref m_hosts; /**< Fully qualified names of the subordinate host objects that
						   * exist under this superordinate domain object */
      eppobject::epp::epp_string_ref m_client_id; /**< Identifier of the sponsoring client */
      eppobject::epp::epp_string_ref m_created_by; /**< Identifier of the client that created the domain */
      eppobject::epp::epp_datetime_ref m_created_date; /**< Date and time of domain object creation */
      eppobject::epp::epp_string_ref m_updated_by; /**< Identifier of the client that last updated the domain */
      eppobject::epp::epp_datetime_ref m_updated_date; /**< Date and time of the most recent domain modification */
      eppobject::epp::epp_datetime_ref m_expiration_date; /**< Date and time identifying the end of the
							   * domain object's registration period */
      eppobject::epp::epp_datetime_ref m_transfer_date; /**< Date and time of the most recent successful domain transfer */
      eppobject::epp::epp_AuthInfo_ref m_auth_info; /**< Authorization information associated with domain */

      /// Null constructor
      epp_DomainInfoRsp() {};
      
      /// Destructor
      virtual ~epp_DomainInfoRsp() {}
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_DomainInfoRsp(const eppobject::epp::epp_Response_ref & _m_rsp,
			const eppobject::epp::epp_string & _m_name,
			const eppobject::epp::epp_roid & _m_roid,
			const epp_domain_status_seq_ref & _m_status,
			const eppobject::epp::epp_string & _m_registrant,
			const epp_domain_contact_seq_ref & _m_contacts,
			const eppobject::epp::epp_string_seq_ref & _m_name_servers,
			const eppobject::epp::epp_string_seq_ref & _m_hosts,
			const eppobject::epp::epp_string & _m_client_id,
			const eppobject::epp::epp_string & _m_created_by,
			const eppobject::epp::epp_datetime & _m_created_date,
			const eppobject::epp::epp_string & _m_updated_by,
			const eppobject::epp::epp_datetime & _m_updated_date,
			const eppobject::epp::epp_datetime & _m_expiration_date,
			const eppobject::epp::epp_datetime & _m_transfer_date,
			const eppobject::epp::epp_AuthInfo_ref & _m_auth_info)
	: epp::epp_GenericRsp(_m_rsp)
	{
	   m_name.ref(new eppobject::epp::epp_string(_m_name));
	   m_roid.ref(new eppobject::epp::epp_roid(_m_roid));
	   m_status = _m_status;
	   m_registrant.ref(new eppobject::epp::epp_string(_m_registrant));
	   m_contacts = _m_contacts;
	   m_name_servers = _m_name_servers;
	   m_hosts = _m_hosts;
	   m_client_id.ref(new eppobject::epp::epp_string(_m_client_id));
	   m_created_by.ref(new eppobject::epp::epp_string(_m_created_by));
	   m_created_date.ref(new eppobject::epp::epp_datetime(_m_created_date));
	   m_updated_by.ref(new eppobject::epp::epp_string(_m_updated_by));
	   m_updated_date.ref(new eppobject::epp::epp_datetime(_m_updated_date));
	   m_expiration_date.ref(new eppobject::epp::epp_datetime(_m_expiration_date));
	   m_transfer_date.ref(new eppobject::epp::epp_datetime(_m_transfer_date));
	   m_auth_info = _m_auth_info;
	};
      
      /// Constructor, using only references
      epp_DomainInfoRsp(const eppobject::epp::epp_Response_ref & _m_rsp,
			const eppobject::epp::epp_string_ref & _m_name,
			const eppobject::epp::epp_roid_ref & _m_roid,
			const epp_domain_status_seq_ref & _m_status,
			const eppobject::epp::epp_string_ref & _m_registrant,
			const epp_domain_contact_seq_ref & _m_contacts,
			const eppobject::epp::epp_string_seq_ref & _m_name_servers,
			const eppobject::epp::epp_string_seq_ref & _m_hosts,
			const eppobject::epp::epp_string_ref & _m_client_id,
			const eppobject::epp::epp_string_ref & _m_created_by,
			const eppobject::epp::epp_datetime_ref & _m_created_date,
			const eppobject::epp::epp_string_ref & _m_updated_by,
			const eppobject::epp::epp_datetime_ref & _m_updated_date,
			const eppobject::epp::epp_datetime_ref & _m_expiration_date,
			const eppobject::epp::epp_datetime_ref & _m_transfer_date,
			const eppobject::epp::epp_AuthInfo_ref & _m_auth_info)
	: epp::epp_GenericRsp(_m_rsp)
	{
	   m_name = _m_name;
	   m_roid = _m_roid;
	   m_status = _m_status;
	   m_registrant = _m_registrant;
	   m_contacts = _m_contacts;
	   m_name_servers = _m_name_servers;
	   m_hosts = _m_hosts;
	   m_client_id = _m_client_id;
	   m_created_by = _m_created_by;
	   m_created_date = _m_created_date;
	   m_updated_by = _m_updated_by;
	   m_updated_date = _m_updated_date;
	   m_expiration_date = _m_expiration_date;
	   m_transfer_date = _m_transfer_date;
	   m_auth_info = _m_auth_info;
	};
   };

   /// Typedef for the ref to the epp_DomainInfoRsp class
   typedef refcnt_ptr<epp_DomainInfoRsp> epp_DomainInfoRsp_ref;
   
}}

#endif
