/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_DOMAINPERIOD_H
#define __EPP_DOMAINPERIOD_H

/** @file epp_DomainPeriod.h
 *  @brief Definition of epp_DomainPeriod and epp_DomainPeriod_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_domaindata.h"

namespace eppobject { namespace domain {

   /// Class describing registration period of the domain object
   class epp_DomainPeriod {
      
    public:
      epp_DomainPeriodUnitType_ref m_unit; /**< Unit type of the domain period */
      eppobject::epp::epp_short_ref m_value; /**< Value of domain period */
      
      /// Null constructor
      epp_DomainPeriod() {};
      
      /// Destructor
      virtual ~epp_DomainPeriod() {};
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_DomainPeriod(const epp_DomainPeriodUnitType & _m_unit,
		       const eppobject::epp::epp_short & _m_value)
	{
	   m_unit.ref(new epp_DomainPeriodUnitType(_m_unit));
	   //lint -e1556
	   // We know that this line is not really suspicious...
	   m_value.ref(new eppobject::epp::epp_short(_m_value));
	   //lint +e1556
	};
      
      /// Constructor, using only references
      epp_DomainPeriod(const epp_DomainPeriodUnitType_ref & _m_unit,
		       const eppobject::epp::epp_short_ref & _m_value)
	{
	   m_unit = _m_unit;
	   m_value = _m_value;
	};
   };

   /// Typedef for the ref to the epp_DomainPeriod class
   typedef refcnt_ptr<epp_DomainPeriod> epp_DomainPeriod_ref;
   
}}

#endif
