/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_DOMAINTRANSFERREQ_H
#define __EPP_DOMAINTRANSFERREQ_H

/** @file epp_DomainTransferReq.h
 *  @brief Definition of epp_DomainTransferReq and epp_DomainTransferReq_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_domaindata.h"
#include "epp_GenericReq.h"
#include "epp_TransferRequest.h"
#include "epp_DomainPeriod.h"

namespace eppobject { namespace domain {

   /// Class containing elements used in domain transfer requests
   /// @see epp_GenericReq for additional information
   class epp_DomainTransferReq : public epp::epp_GenericReq {

    public:
      eppobject::epp::epp_TransferRequest_ref m_trans; /**< Common and generic transfer request element */
      eppobject::epp::epp_string_ref m_name; /**< Fully qualified name of the domain object to be transferred */
      epp_DomainPeriod_ref m_period; /**< Number of units to be added to the registration period of the domain
				       * object at completion of the transfer process */
      
      /// Null constructor
      epp_DomainTransferReq() {};
      
      /// Destructor
      virtual ~epp_DomainTransferReq() {}
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_DomainTransferReq(const eppobject::epp::epp_Command_ref & _m_cmd,
			    const eppobject::epp::epp_TransferRequest_ref & _m_trans,
			    const eppobject::epp::epp_string & _m_name,
			    const epp_DomainPeriod_ref & _m_period)
	: epp::epp_GenericReq(_m_cmd)
	{
	   m_trans = _m_trans;
	   m_name.ref(new eppobject::epp::epp_string(_m_name));
	   m_period = _m_period;
	};
      
      /// Constructor, using only references
      epp_DomainTransferReq(const eppobject::epp::epp_Command_ref & _m_cmd,
			    const eppobject::epp::epp_TransferRequest_ref & _m_trans,
			    const eppobject::epp::epp_string_ref & _m_name,
			    const epp_DomainPeriod_ref & _m_period)
	: epp::epp_GenericReq(_m_cmd)
	{
	   m_trans = _m_trans;
	   m_name = _m_name;
	   m_period = _m_period;
	};
   };

   /// Typedef for the ref to the epp_DomainTransferReq class
   typedef refcnt_ptr<epp_DomainTransferReq> epp_DomainTransferReq_ref;
   
}}

#endif
