/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_DOMAINUPDATEREQ_H
#define __EPP_DOMAINUPDATEREQ_H

/** @file epp_DomainUpdateReq.h
 *  @brief Definition of epp_DomainUpdateReq and epp_DomainUpdateReq_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_domaindata.h"
#include "epp_GenericReq.h"
#include "epp_DomainUpdateAddRemove.h"
#include "epp_DomainUpdateChange.h"

namespace eppobject { namespace domain {
   
   /// Class containing elements used in domain update requests
   /// @see epp_GenericReq for additional information
   class epp_DomainUpdateReq : public epp::epp_GenericReq {
      
    public:
      eppobject::epp::epp_string_ref m_name; /**< Fully qualified name of the domain object to be updated */
      epp_DomainUpdateAddRemove_ref m_add; /**< Attribute values to be added to the object */
      epp_DomainUpdateChange_ref m_change; /**< Object attribute values to be changed */
      epp_DomainUpdateAddRemove_ref m_remove; /**< Attribute values to be removed from the object */
      
      /// Null constructor
      epp_DomainUpdateReq() {};

      /// Destructor
      virtual ~epp_DomainUpdateReq() {}
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_DomainUpdateReq(const eppobject::epp::epp_Command_ref & _m_cmd,
			  const eppobject::epp::epp_string & _m_name,
			  const epp_DomainUpdateAddRemove_ref & _m_add,
			  const epp_DomainUpdateChange_ref & _m_change,
			  const epp_DomainUpdateAddRemove_ref & _m_remove)
	: epp::epp_GenericReq(_m_cmd)
	{
	   m_name.ref(new eppobject::epp::epp_string(_m_name));
	   m_add = _m_add;
	   m_change = _m_change;
	   m_remove = _m_remove;
	};
      
      /// Constructor, using only references
      epp_DomainUpdateReq(const eppobject::epp::epp_Command_ref & _m_cmd,
			  const eppobject::epp::epp_string_ref & _m_name,
			  const epp_DomainUpdateAddRemove_ref & _m_add,
			  const epp_DomainUpdateChange_ref & _m_change,
			  const epp_DomainUpdateAddRemove_ref & _m_remove)
	: epp::epp_GenericReq(_m_cmd)
	{
	   m_name = _m_name;
	   m_add = _m_add;
	   m_change = _m_change;
	   m_remove = _m_remove;
	};
   };
   
   /// Typedef for the ref to the epp_DomainUpdateReq class
   typedef refcnt_ptr<epp_DomainUpdateReq> epp_DomainUpdateReq_ref;
   
}}

#endif
