/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_EXT_RESULT_VALUE_H
#define __EPP_EXT_RESULT_VALUE_H

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/data/epp_ExtResultValue.h,v 1.3 2003/07/18 20:21:19 tubadanm Exp $ */

/** @file epp_ExtResultValue.h
 *  @brief Definition of epp_ExtResultValue and epp_ExtResultValue_ref
 *  @author Daniel Manley
 */

#include "epp_eppdata.h"
#include "epp_ResultValue.h"

namespace eppobject { namespace epp {
   
   /// Class containing elements used to describe success/failure of
   /// a command in server responses
   class epp_ExtResultValue {
      
    public:
      
      epp_ResultValue_ref m_value; /**< The xml element from the request which caused the error */
      epp_string_ref m_reason; /**< The free text reason for the error */
      
      /// Null constructor
      epp_ExtResultValue() {};
      
      /// Destructor
      virtual ~epp_ExtResultValue() {};
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_ExtResultValue(const epp_ResultValue & _m_value,
		         const epp_string & _m_reason)
	{
	   m_value.ref(new epp_ResultValue(_m_value));
	   m_reason.ref(new epp_string(_m_reason));
	};
      
      /// Constructor, using only references
      epp_ExtResultValue(const epp_ResultValue_ref & _m_value,
  		         const epp_string_ref & _m_reason)
	{
	   m_value = _m_value;
	   m_reason = _m_reason;
	};
   };
   
   /// Typedef for the ref to the epp_ExtResultValue class
   typedef refcnt_ptr<epp_ExtResultValue> epp_ExtResultValue_ref;
   std::ostream& operator<<(std::ostream &stream, const epp_ExtResultValue_ref & extresvalue);
   std::ostream& operator<<(std::ostream &stream, const epp_ExtResultValue & extresvalue);
   
}}

#endif
