/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_GREETING_H
#define __EPP_GREETING_H

/** @file epp_Greeting.h
 *  @brief Definition of epp_Greeting and epp_Greeting_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"
#include "epp_ServiceMenu.h"
#include "epp_DataCollectionPolicy.h"

namespace eppobject { namespace epp {
   
   /// Class describing the server and its supported services
   class epp_Greeting {
      
    public:
      epp_string_ref m_server_id; /**< Name of the server */
      epp_datetime_ref m_server_date; /**< Current date and time in UTC of the server */
      epp_ServiceMenu_ref m_svc_menu; /**< Menu identifying the services supported by the server */
      epp_DataCollectionPolicy_ref m_dcp; /**< Elements used to describe the server's policy for data
					   *   collection and management */
      
      /// Null constructor
      epp_Greeting() {};
      
      /// Destructor
      virtual ~epp_Greeting() {}
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_Greeting(const epp_string & _m_server_id,
		   const epp_datetime & _m_server_date,
		   const epp_ServiceMenu_ref & _m_svc_menu,
		   const epp_DataCollectionPolicy_ref & _m_dcp)
	{
	   m_server_id.ref(new epp_string(_m_server_id));
	   m_server_date.ref(new epp_datetime(_m_server_date));
	   m_svc_menu = _m_svc_menu;
	   m_dcp = _m_dcp;
	};
      
      /// Constructor, using only references
      epp_Greeting(const epp_string_ref & _m_server_id,
		   const epp_datetime_ref & _m_server_date,
		   const epp_ServiceMenu_ref & _m_svc_menu,
		   const epp_DataCollectionPolicy_ref & _m_dcp)
	{
	   m_server_id = _m_server_id;
	   m_server_date = _m_server_date;
	   m_svc_menu = _m_svc_menu;
	   m_dcp = _m_dcp;
	};
   };
   
   /// Typedef for the ref to the epp_Greeting class
   typedef refcnt_ptr<epp_Greeting> epp_Greeting_ref;
   
}}

#endif
