/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_HOSTADDRESS_H
#define __EPP_HOSTADDRESS_H

/** @file epp_HostAddress.h
 *  @brief Definition of epp_HostAddress and epp_HostAddress_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_hostdata.h"

namespace eppobject { namespace host {
   
   /// Class containing information about a host address
   class epp_HostAddress {

    public:
      epp_HostAddressType_ref m_type; /**< Type of host address */
      eppobject::epp::epp_string_ref m_ip;/**< Host IP address */
      
      /// Null constructor
      epp_HostAddress() {};
      
      /// Destructor
      virtual ~epp_HostAddress() {};
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_HostAddress(const epp_HostAddressType & _m_type,
		      const eppobject::epp::epp_string & _m_ip)
	{
	   m_type.ref(new epp_HostAddressType(_m_type));
	   m_ip.ref(new eppobject::epp::epp_string(_m_ip));
	};
      
      /// Constructor, using only references
      epp_HostAddress(const epp_HostAddressType_ref & _m_type,
		      const eppobject::epp::epp_string_ref & _m_ip)
	{
	   m_type = _m_type;
	   m_ip = _m_ip;
	};
   };

   /// Typedef for the ref to the epp_HostAddress class
   typedef refcnt_ptr<epp_HostAddress> epp_HostAddress_ref;
   
}}

#endif
