/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_HOSTCREATERSP_H
#define __EPP_HOSTCREATERSP_H

/** @file epp_HostCreateRsp.h
 *  @brief Definition of epp_HostCreateRsp and epp_HostCreateRsp_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_hostdata.h"
#include "epp_GenericRsp.h"

namespace eppobject { namespace host {
   
   /// Class containing elements used in host create responses
   /// @see epp_GenericRsp for additional information
   class epp_HostCreateRsp : public epp::epp_GenericRsp {
      
    public:
      eppobject::epp::epp_string_ref m_name; /**< Fully qualified name of the created host object */
      eppobject::epp::epp_datetime_ref m_creation_date; /**< Date and time of host object creation */
      
      /// Null constructor
      epp_HostCreateRsp() {};

      /// Destructor
      virtual ~epp_HostCreateRsp() {}
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_HostCreateRsp(const eppobject::epp::epp_Response_ref & _m_rsp,
			const eppobject::epp::epp_string & _m_name,
			const eppobject::epp::epp_datetime & _m_creation_date)
	: epp::epp_GenericRsp(_m_rsp)
	{
	   m_name.ref(new eppobject::epp::epp_string(_m_name));
	   m_creation_date.ref(new eppobject::epp::epp_datetime(_m_creation_date));
	};
      
      /// Constructor, using only references
      epp_HostCreateRsp(const eppobject::epp::epp_Response_ref & _m_rsp,
			const eppobject::epp::epp_string_ref & _m_name,
			const eppobject::epp::epp_datetime_ref & _m_creation_date)
	: epp::epp_GenericRsp(_m_rsp)
	{
	   m_name = _m_name;
	   m_creation_date = _m_creation_date;
	};
   };
   
   /// Typedef for the ref to the epp_HostCreateRsp class
   typedef refcnt_ptr<epp_HostCreateRsp> epp_HostCreateRsp_ref;
   
}}

#endif
