/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_LOGOUTREQ_H
#define __EPP_LOGOUTREQ_H

/** @file epp_LogoutReq.h
 *  @brief Definition of epp_LogoutReq and epp_LogoutReq_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"
#include "../epp_Extension.h"

namespace eppobject { namespace epp {
   
   /// Class containing elements used in logout requests
   class epp_LogoutReq {
      
    public:
      epp_extension_ref_seq_ref m_extensions; /**< Extension elements used for server-defined command extensions */
      epp_trid_ref m_client_trid; /**< Client transaction ID element that uniquely identifies the command to the client */
      
      /// Null constructor
      epp_LogoutReq() {};
      
      /// Destructor
      virtual ~epp_LogoutReq() {}

      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      /// @deprecated
      epp_LogoutReq(const epp_Extension_ref & _m_ext,
		    const epp_trid & _m_client_trid)
	{
	   m_extensions = new(epp_extension_ref_seq);
	   m_extensions->push_back(_m_ext);
	   m_client_trid.ref(new epp_trid(_m_client_trid));
	};
      
      /// Constructor, using only references
      /// @deprecated
      epp_LogoutReq(const epp_Extension_ref & _m_ext,
		    const epp_trid_ref & _m_client_trid)
	{
	   m_extensions = new(epp_extension_ref_seq);
	   m_extensions->push_back(_m_ext);
	   m_client_trid = _m_client_trid;
	};

      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_LogoutReq(const epp_extension_ref_seq_ref & _m_extensions,
		    const epp_trid & _m_client_trid)
	{
	   m_extensions = _m_extensions;
	   m_client_trid.ref(new epp_trid(_m_client_trid));
	};
      
      /// Constructor, using only references
      epp_LogoutReq(const epp_extension_ref_seq_ref & _m_extensions,
		    const epp_trid_ref & _m_client_trid)
	{
	   m_extensions = _m_extensions;
	   m_client_trid = _m_client_trid;
	};
   };
   
   /// Typedef for the ref to the epp_LogoutReq class
   typedef refcnt_ptr<epp_LogoutReq> epp_LogoutReq_ref;
   
}}

#endif
