/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_POLLREQ_H
#define __EPP_POLLREQ_H

/** @file epp_PollReq.h
 *  @brief Definition of epp_PollReq and epp_PollReq_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"
#include "epp_GenericReq.h"

namespace eppobject { namespace epp {
   
   /// Class containing elements used in poll requests
   /// @see epp_GenericReq for additional information
   class epp_PollReq : public epp_GenericReq {
      
    public:
      epp_PollOpType_ref m_op; /**< Operation type, e.g., "ack"(knowledge) and "req"(uest) */
      epp_string_ref m_msgID; /**< Message ID when acknowledging receipt of a message */
      
      /// Null constructor
      epp_PollReq() {};
      
      /// Destructor
      virtual ~epp_PollReq() {}
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_PollReq(const epp_Command_ref & _m_cmd,
		  const epp_PollOpType & _m_op,
		  const epp_string & _m_msgID)
	: epp_GenericReq(_m_cmd)
	{
	   m_op.ref(new epp_PollOpType(_m_op));
	   m_msgID.ref(new epp_string(_m_msgID));
	};
      
      /// Constructor, using only references
      epp_PollReq(const epp_Command_ref & _m_cmd,
		  const epp_PollOpType_ref & _m_op,
		  const epp_string_ref & _m_msgID)
	: epp_GenericReq(_m_cmd)
	{
	   m_op = _m_op;
	   m_msgID = _m_msgID;
	};
   };

   /// Typedef for the ref to the epp_PollReq class
   typedef refcnt_ptr<epp_PollReq> epp_PollReq_ref;
   
}}

#endif
