/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_POLLRESFACTORY_H
#define __EPP_POLLRESFACTORY_H

/** @file epp_PollResFactory.h
 *  @brief Definition of epp_PollResFactory and epp_PollResFactory_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"
#include "epp_PollResData.h"
#include <map>

namespace eppobject { namespace epp {
   
   /// Class handling all the different poll resData responses
   class epp_PollResFactory {
      
    private:
      /// Typedef of the epp_PollResData factory used in the class
      typedef std::map<epp_string, epp_PollResData * (*)(), std::less<epp_string> > FactoryMap;
      /// The epp_PollResData factory
      static FactoryMap fm;
      
    public:
      
      /// Null constructor
      epp_PollResFactory() {};
      
      /// Destructor
      virtual ~epp_PollResFactory() {};
      
      /** Checks if the resData factory map has an element corresponding to the namespaceURI
       * and the element, and returns an object of the corresponding resData class if it is
       * of known type
       * @param namespaceURI Namespace URI of the poll resData element
       * @param element Local name of the poll resData element
       * @return NULL, or a pointer to the corresponding resData object
       */
      static epp_PollResData* create(const std::string & namespaceURI,
				     const std::string & element);
      
      /** Adds a possible poll resData element to the epp_PollResData factory map
       * @param namespaceURI Namespace URI of the poll resData element
       * @param element Local name of the poll resData element
       * @param func Name of function used to create an object of the corresponding poll resData class
       */
      static void addClass(const std::string & namespaceURI,
			   const std::string & element,
			   epp_PollResData * (* func)());
      
   };
   
   /// Typedef for the ref to the epp_PollResFactory class
   typedef refcnt_ptr<epp_PollResFactory> epp_PollResFactory_ref;
   
}}

#endif
