/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_RESPONSE_H
#define __EPP_RESPONSE_H

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/data/epp_Response.h,v 1.13 2003/07/18 20:21:19 tubadanm Exp $ */

/** @file epp_Response.h
 *  @brief Definition of epp_Response and epp_Response_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"
#include "epp_Result.h"
#include "epp_MessageQueue.h"
#include "epp_TransID.h"

namespace eppobject { namespace epp {
   
   /// Class containing elements used in the response from the server
   class epp_Response {
      
    public:
      
      epp_result_seq_ref m_results; /**< Structure to document the success or failure of command execution */
      epp_MessageQueue_ref m_message_queue; /**< Information about service messages queued for client retrieval */
      epp_xml_string_seq_ref m_ext_strings; /**< Extension element used for server-defined response extensions */
      epp_TransID_ref m_trans_id; /**< Transaction identifier assigned by the server to the command */
      
      /// Null constructor
      epp_Response() {};
      
      /// Destructor
      virtual ~epp_Response() {};
      
      /// Constructor, using only references
      epp_Response(const epp_result_seq_ref & _m_results,
		   const epp_MessageQueue_ref & _m_message_queue,
		   const epp_xml_string_seq_ref & _m_ext_strings,
		   const epp_TransID_ref & _m_trans_id)
	{
	   m_results = _m_results;
	   m_message_queue = _m_message_queue;
	   m_ext_strings = _m_ext_strings;
	   m_trans_id = _m_trans_id;
	};
   };

   /// Typedef for the ref to the epp_Response class
   typedef refcnt_ptr<epp_Response> epp_Response_ref;
   /// stream output of a response reference pointer
   std::ostream& operator<<(std::ostream &stream, const epp_Response_ref & response);
   std::ostream& operator<<(std::ostream &stream, const epp_Response & response);
   
}}

#endif
