/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_TRANSID_H
#define __EPP_TRANSID_H

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/data/epp_TransID.h,v 1.11 2003/07/18 20:21:19 tubadanm Exp $ */

/** @file epp_TransID.h
 *  @brief Definition of epp_TransID and epp_TransID_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"

namespace eppobject { namespace epp {
   
   /// Class containing transaction ID elements assigned by the server
   /// to the command for which the response is being returned
   class epp_TransID {
      
    public:
      epp_trid_ref m_client_trid; /**< Client transaction ID associated with the command */
      epp_trid_ref m_server_trid; /**< Server transaction ID assigned by and unique to the server */
      
      /// Null constructor
      epp_TransID() {};
      
      /// Destructor
      virtual ~epp_TransID() {};      
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_TransID(const epp_trid & _m_client_trid,
		  const epp_trid & _m_server_trid)
	{
	   m_client_trid.ref(new epp_trid(_m_client_trid));
	   m_server_trid.ref(new epp_trid(_m_server_trid));
	};
      
      /// Constructor, using only references
      epp_TransID(const epp_trid_ref & _m_client_trid,
		  const epp_trid_ref & _m_server_trid)
	{
	   m_client_trid = _m_client_trid;
	   m_server_trid = _m_server_trid;
	};
   };
   
   /// Typedef for the ref to the epp_TransID class
   typedef refcnt_ptr<epp_TransID> epp_TransID_ref;
   std::ostream& operator<<(std::ostream &stream, const epp_TransID_ref & transid);
   std::ostream& operator<<(std::ostream &stream, const epp_TransID & transid);
   
}}

#endif
