/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_CONTACTXMLBASE_H
#define __EPP_CONTACTXMLBASE_H

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/data/epp_contactXMLbase.h,v 1.11 2003/07/18 20:21:19 tubadanm Exp $ */

/** @file epp_contactXMLbase.h
 *  @brief Declaration of XML helper-functions for contact objects
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_contactdata.h"
#include "epp_eppXMLbase.h"
#include "epp_ContactNameAddress.h"

namespace eppobject { namespace contact {
   
   /** Writes attributes that identify the contact namespace and the location 
    *  of the contact schema to the output object
    * @param outputobject The output object to write to
    */
   void setCommandSchemaAttributes(eppobject::epp::EPP_output & outputobject);
   
   /** Writes attributes that identify the contact namespace and the location
    *  of the contact schema to the output object
    * @param outputobject The output object to write to
    * @param element The epp_ContactNameAddress element to add
    * @param tag The encapsulating XML-tag to use
    * @param update There are checks for missing elements only if this
    *        value is set to true ("update" as opposed to "create")
    * @throw epp_XMLException If mandatory data is missing in the epp_ContactNameAddress object
    */
   void addAddressElement(eppobject::epp::EPP_output & outputobject,
			  const epp_ContactNameAddress_ref & element,
			  const eppobject::epp::epp_string & tag = "contact:postalInfo",
			  const eppobject::epp::epp_boolean & update = false);


   /** Parses an address dom_ptr and creates the corresonding epp_ContactNameAddress object
    * @param adrNode The address dom_ptr
    * @return The corresponding epp_ContactNameAddress object
    * @throw epp_XMLException If it is unable find the correct XML-elements in the Node
    * @note The function might have problems with different namespaces
    */
   epp_ContactNameAddress parseAddress(const domtools::dom_ptr & adrNode);

   /** Returns the corresponsing epp_string text for the given epp_ContactStatusType-enum
    * @param statusType The contact status type as an epp_ContactStatusType-enum
    * @return The contact status type as an epp_string
    * @throw epp_XMLException If statusType is not a valid epp_ContactStatusType
    */
   eppobject::epp::epp_string returnStatusType(const epp_ContactStatusType & statusType);   
   
   /** Returns the corresponsing epp_ContactStatusType for the given epp_string
    * @param statusType The contact status type as an epp_string
    * @return The contact status type as an epp_ContactStatusType-enum
    * @throw epp_XMLException If statusType is not a valid epp_ContactStatusType
    */
   epp_ContactStatusType returnStatusEnumType(const eppobject::epp::epp_string & statusType);

   
   
   /*********************************23**********************************
    *                                                                   *
    *           Only deprecated functions below this, please...         *
    *                                                                   *
    *********************************23**********************************/
   
   
   /** Parses an address DOM_Node and creates the corresonding epp_ContactNameAddress object
    * @param adrNode The address DOM_Node
    * @return The corresponding epp_ContactNameAddress object
    * @throw epp_XMLException If it is unable find the correct XML-elements in the Node
    * @deprecated Replaced by parseAddress(const domtools::dom_ptr & adrNode);
    */
   epp_ContactNameAddress parseAddress(const DOM_Node & adrNode);
   
}}
#endif
