/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_DCP_EXPIRY_H
#define __EPP_DCP_EXPIRY_H

/** @file epp_dcpExpiry.h
 *  @brief Definition of epp_dcpExpiry and epp_dcpExpiry_ref
 *  @author Daniel Manley
 */

#include "epp_eppdata.h"
#include "epp_ResultValue.h"

namespace eppobject { namespace epp {
   
   /// Class containing elements used to describe success/failure of
   /// a command in server responses
   class epp_dcpExpiry {
      
    public:
      
      epp_dcpExpiryType_ref m_type;  /**< The DCP expiry type */
      epp_string_ref m_value; /**< The expiry date of the DCP -- there is not strict format for this field */
      
      /// Null constructor
      epp_dcpExpiry() {};
      
      /// Destructor
      virtual ~epp_dcpExpiry() {};
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_dcpExpiry(const epp_dcpExpiryType & _m_type,
		    const epp_string & _m_value)
	{
	   m_type.ref(new epp_dcpExpiryType(_m_type));
	   m_value.ref(new epp_string(_m_value));
	};
      
      /// Constructor, using only references
      epp_dcpExpiry(const epp_dcpExpiryType & _m_type,
  		    const epp_string_ref & _m_value)
	{
	   m_type = _m_type;
	   m_value = _m_value;
	};
   };
   
   /// Typedef for the ref to the epp_dcpExpiry class
   typedef refcnt_ptr<epp_dcpExpiry> epp_dcpExpiry_ref;
   
}}

#endif
