/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_DOMAINXMLBASE_H
#define __EPP_DOMAINXMLBASE_H

/** @file epp_domainXMLbase.h
 *  @brief Declaration of XML helper-functions for domain objects
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_domaindata.h"
#include "epp_eppXMLbase.h"


namespace eppobject { namespace domain {

   /** Writes attributes that identify the domain namespace and the location 
    *  of the domain schema to the output object
    * @param outputobject The output object to write to
    */
   void setCommandSchemaAttributes(eppobject::epp::EPP_output & outputobject);
   
   /** Returns the corresponsing epp_string text for the given epp_DomainPeriodUnitType-enum
    * @param unittype The domain period unit type as an epp_DomainPeriodUnitType-enum
    * @return The domain period unit type as an epp_string
    * @throw epp_XMLException If unittype is not a valid epp_DomainPeriodUnitType
    */
   eppobject::epp::epp_string returnPeriodUnitType(const epp_DomainPeriodUnitType & unittype);
  
   /** Returns the corresponsing epp_string text for the given epp_DomainContactType-enum
    * @param contacttype The contact type as an epp_DomainContactType-enum
    * @return The contact type as an epp_string
    * @throw epp_XMLException If contacttype is not a valid epp_DomainContactType
    */
   eppobject::epp::epp_string returnContactType(const epp_DomainContactType & contacttype);
   
   /** Returns the corresponsing epp_DomainContactType for the given epp_string
    * @param contactype The contact type as an epp_string
    * @return The contact type as an epp_DomainContactType-enum
    * @throw epp_XMLException If contacttype is not a valid epp_DomainContactType
    */
   epp_DomainContactType returnContactEnumType(const eppobject::epp::epp_string & contacttype);
   
   /** Returns the corresponsing epp_string text for the given epp_DomainStatusType-enum
    * @param statusType The domain status type as an epp_DomainStatusType-enum
    * @return The domain status type as an epp_string
    * @throw epp_XMLException If statusType is not a valid epp_DomainStatusType
    */
   eppobject::epp::epp_string returnStatusType(const epp_DomainStatusType & statusType);
   
   /** Returns the corresponsing epp_DomainStatusType for the given epp_string
    * @param statusType The domain status type as an epp_string
    * @return The domain status type as an epp_DomainStatusType-enum
    * @throw epp_XMLException If statusType is not a valid epp_DomainStatusType
    */
   epp_DomainStatusType returnStatusEnumType(const eppobject::epp::epp_string & statusType);
   
   /** Returns the corresponsing epp_DomainHostsType for the given epp_string
    * @param hostsType The domain hosts type as an epp_string
    * @return The domain hosts type as an epp_DomainHostsType-enum
    * @throw epp_XMLException If hostsType is not a valid epp_DomainHostsType
    */
   epp_DomainHostsType returnDomainHostsEnumType(const eppobject::epp::epp_string & hostsType);

   /** Returns the corresponsing epp_string text for the given epp_DomainHostsType-enum
    * @param hostsType The domain hosts type as an epp_DomainHostsType-enum
    * @return The domain hosts type as an epp_string
    * @throw epp_XMLException If hostsType is not a valid epp_DomainHostsType
    */
   eppobject::epp::epp_string returnDomainHostsType(const epp_DomainHostsType & hostsType);

}}
#endif
