/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_DOMAINDATA_H
#define __EPP_DOMAINDATA_H

/** @file epp_domaindata.h
 *  @brief Declaration of enums/typedefs used in domain objects
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"
#include <vector>


namespace eppobject { namespace domain {

   /******************************************
    *                                        *
    *              Enumerations              *
    *                                        *
    ******************************************/
   
   /// Status types allowed for a domain object
   enum epp_DomainStatusType {
      CLIENT_DELETE_PROHIBITED,
      CLIENT_HOLD,
      CLIENT_RENEW_PROHIBITED,
      CLIENT_TRANSFER_PROHIBITED,
      CLIENT_UPDATE_PROHIBITED,
      INACTIVE,
      OK,
      PENDING_CREATE,
      PENDING_DELETE,
      PENDING_RENEW,
      PENDING_TRANSFER,
      PENDING_UPDATE,
      SERVER_DELETE_PROHIBITED,
      SERVER_HOLD,
      SERVER_RENEW_PROHIBITED,
      SERVER_TRANSFER_PROHIBITED,
      SERVER_UPDATE_PROHIBITED
   };
   
   /// Typedef for the ref to the #epp_DomainStatusType enum
   typedef refcnt_ptr<epp_DomainStatusType> epp_DomainStatusType_ref;
   
   /// Registration period unit types allowed for a domain
   enum epp_DomainPeriodUnitType {
      YEAR,
      MONTH
   };
   
   /// Typedef for the ref to the #epp_DomainPeriodUnitType enum
   typedef refcnt_ptr<epp_DomainPeriodUnitType> epp_DomainPeriodUnitType_ref;

   /// Domain contact types allowed for a domain
   enum epp_DomainContactType {
      ADMIN,
      BILLING,
      TECH
   };
   
   /// Typedef for the ref to the #epp_DomainContactType enum
   typedef refcnt_ptr<epp_DomainContactType> epp_DomainContactType_ref;
   
   /// Domain hosts listing types (used in info requests)
   enum epp_DomainHostsType {
      ALL,
      DEL,
      NONE,
      SUB
   };
   
   /// Typedef for the ref to the #epp_DomainHostsType enum
   typedef refcnt_ptr<epp_DomainHostsType> epp_DomainHostsType_ref;
   
   
   /******************************************
    *                                        *
    *              Pure typedefs             *
    *                                        *
    ******************************************/
   
   class epp_DomainStatus; // Forward declaration...
   
   /// Typedef for a sequence of #epp_DomainStatus elements
   typedef std::vector<epp_DomainStatus> epp_domain_status_seq;
   
   /// Typedef for the ref to #epp_domain_status_seq
   typedef refcnt_ptr<epp_domain_status_seq> epp_domain_status_seq_ref;
   
   class epp_DomainContact; // Forward declaration...
   
   /// Typedef for a sequence of #epp_DomainContact elements
   typedef std::vector<epp_DomainContact> epp_domain_contact_seq;
   
   /// Typedef for the ref to #epp_domain_contact_seq
   typedef refcnt_ptr<epp_domain_contact_seq> epp_domain_contact_seq_ref;

}}
#endif
