/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/epp_ContactCreate.cc,v 1.16 2003/07/18 20:21:19 tubadanm Exp $ */

#include "config.h"
#include "epp_ContactCreate.h"
#include <domtools/dom_output.h>
#include <domtools/dom_wrapper.h>
#include "data/epp_contactXMLbase.h"
#include "data/epp_XMLException.h"


using namespace std;
using namespace domtools;
using namespace eppobject::contact;
using namespace eppobject::epp;


void epp_ContactCreate::setRequestData(const epp_ContactCreateReq & parms)
{
   m_request.ref(new epp_ContactCreateReq(parms));
}

epp_ContactCreateRsp_ref epp_ContactCreate::getResponseData() const
{
   return m_response;
}


epp_string epp_ContactCreate::toXML()
{
   if(m_request == NULL || m_request->m_cmd == NULL || m_request->m_id == NULL) {
      throw epp_XMLException(__FILE__, __LINE__, "Missing data in request");
   }
   
   EPP_output xml_request;
   xml_request.setWhitespace(Whitespace);
   
   createStandardXMLHeader(xml_request);
   
   xml_request.beginTag("command");
   
   xml_request.beginTag("create");

   xml_request.beginTag("contact:create");

   setCommandSchemaAttributes(xml_request);

   xml_request.putTag("contact:id",*m_request->m_id);
   
   if( m_request->m_ascii_address != NULL ||
       m_request->m_i15d_address  != NULL ) {

      // this is for backward compatibility with older EPP versions

      if(m_request->m_ascii_address != NULL) {
         m_request->m_ascii_address->m_type.ref(new epp_ContactPostalInfoType(INT));
         addAddressElement(xml_request, m_request->m_ascii_address);
      } // if "m_ascii_address"

      if(m_request->m_i15d_address != NULL) {
         m_request->m_ascii_address->m_type.ref(new epp_ContactPostalInfoType(LOC));
         addAddressElement(xml_request, m_request->m_i15d_address);
      } // if "m_i15d_address"

   } else if ( m_request->m_addresses != NULL &&
               m_request->m_addresses->size() > 0 ) {
      // now let's deal with the newer addresses list...
      
      for(epp_ContactNameAddress_seq::iterator i = m_request->m_addresses->begin();
          i != m_request->m_addresses->end(); i++)
        {
         epp_ContactNameAddress_ref cnar;
         cnar.ref(new epp_ContactNameAddress(*i));
         addAddressElement(xml_request, cnar);
        }
      
   }
   if(m_request->m_voice != NULL) {
      
      if(m_request->m_voice->m_value == NULL) {
	 throw epp_XMLException(__FILE__, __LINE__, "Missing data in ContactPhone-element (voice)");
      }
      
      xml_request.beginTag("contact:voice");
      
      if(m_request->m_voice->m_extension != NULL) {
	 xml_request.putAttribute("x", *m_request->m_voice->m_extension);
      }
      
      xml_request.putCDATA(*m_request->m_voice->m_value);
      xml_request.endTag("contact:voice");
   } // if "m_voice"
   
   if(m_request->m_fax != NULL) {
      
      if(m_request->m_fax->m_value == NULL) {
	 throw epp_XMLException(__FILE__, __LINE__, "Missing data in ContactPhone-element (fax)");
      }
      
      xml_request.beginTag("contact:fax");
      if(m_request->m_fax->m_extension != NULL) {
	 xml_request.putAttribute("x", *m_request->m_fax->m_extension);
      }
      xml_request.putCDATA(*m_request->m_fax->m_value);
      xml_request.endTag("contact:fax");
   } // if "m_fax"
   

   if(m_request->m_email != NULL) {
      xml_request.putTag("contact:email", *m_request->m_email);
   } // if "m_email"
   
   if(m_request->m_auth_info != NULL) {
      addAuthInfoElement(xml_request, m_request->m_auth_info, "contact:authInfo");
   } // if "m_auth_info"
   
   xml_request.endTag("contact:create");
   
   xml_request.endTag("create");
   
   addExtensionElements(xml_request, m_request->m_cmd->m_extentions);
   
   if(m_request->m_cmd->m_client_trid != NULL) {
      xml_request.putTag("clTRID",*m_request->m_cmd->m_client_trid);
   } // if "m_cmd->m_client_trid"
   
   xml_request.endTag("command");
   
   createStandardXMLFooter(xml_request);
   
   return xml_request.getString();  
}


void epp_ContactCreate::fromXML(const eppobject::epp::epp_string & xml)
{
   dom_ptr doc = createDOMDocument(xml);
   
   dom_ptr responseNode = dom_ptr(doc->getTag("response"));
   
   if(responseNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find response-element");
   }
   
   m_response.ref(new epp_ContactCreateRsp());

   epp_Response_ref res = createGenericResponse(responseNode);
   m_response->m_rsp = res;
   
   dom_ptr creDataNode = dom_ptr(getresDataInfo(responseNode, "creData"));

   if(creDataNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find creData-element");
   }

   dom_ptr creChildNode = dom_ptr(creDataNode->getFirstChildElement());
   
   string elemPrefix = creDataNode->getPrefix();
   
   while(!creChildNode->isNull()) {
      string nodeName = creChildNode->getNodeName();
      
      if(nodeName == string(elemPrefix + ":id")) {
	 m_response->m_id.ref(new epp_string(creChildNode->getNodeData()));
//	 cout << "id: " << *m_response->m_id << endl;
      } // if "id"
      
      else if(nodeName == string(elemPrefix + ":crDate")) {
	 m_response->m_creation_date.ref(new epp_datetime(creChildNode->getNodeData()));
//	 cout << "crDate: " << *m_response->m_creation_date << endl;
      } // if "crDate"
	
      creChildNode = dom_ptr(creChildNode->getNextSiblingElement());
   } // while "creChildNode"
   
}
