/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_CONTACTUPDATE_H
#define __EPP_CONTACTUPDATE_H

/** @file epp_ContactUpdate.h
 *  @brief Definition of epp_ContactUpdate and epp_ContactUpdate_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "data/epp_ContactUpdateReq.h"
#include "data/epp_ContactUpdateRsp.h"
#include "epp_Action.h"
#include "data/epp_eppXMLbase.h"

namespace eppobject { namespace contact {

   /// Interface class for the EPP contact <update> command
   class epp_ContactUpdate : public eppobject::epp::epp_Action {
      
    private:
      epp_ContactUpdateReq_ref m_request; /**< Request to send to the server */
      epp_ContactUpdateRsp_ref m_response; /**< Response received from server */
      
      /** Helper function putting the add/rem element into the request output object
       * @param outputobject Outputobject to put element into
       * @param element Add/Remove object to process
       * @param mode The tag-name to use (i.e., add or remove)
       */
      void addRemAddElement(eppobject::epp::EPP_output & outputobject,
			    const epp_ContactUpdateAddRemove_ref & element,
			    const eppobject::epp::epp_string & mode) const;
      
      /** Helper function putting the change element into the request output object
       * @param outputobject Outputobject to put element into
       * @param element Change object to process
       */
      void addChangeElement(eppobject::epp::EPP_output & outputobject,
			    const epp_ContactUpdateChange_ref & element) const;
      
    public:
      
      /// Null constructor
      epp_ContactUpdate() {};

      /// Destructor
      virtual ~epp_ContactUpdate() {};
      
      /** Function setting the object's request element
       * @param parms Request element to use
       */
      void setRequestData(const epp_ContactUpdateReq & parms);
      
      /** Function returning the object's response element
       * @return Response element of object
       */
      epp_ContactUpdateRsp_ref getResponseData() const;
      
      /** Function that takes the request data object, parses it, and returns
       *  the corresponding correctly formatted EPP xml request as an
       *  eppobject::epp#epp_string
       * @return The request xml string
       * @throw eppobject::epp::epp_XMLException If request object is missing critical data
       */
      virtual eppobject::epp::epp_string toXML();
      
      /** Function that takes the response xml eppobject::epp#epp_string, parses it,
       *  and puts elements into the class' response data object.
       * @param xml The response xml string
       * @throw eppobject::epp::epp_XMLException If response string is missing critical data
       */
      virtual void fromXML(const eppobject::epp::epp_string & xml);
   };
   
   /// Typedef for the ref to the epp_ContactUpdate class
   typedef refcnt_ptr<epp_ContactUpdate> epp_ContactUpdate_ref;
   
}}

#endif
