/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#include "config.h"
#include "ltoa.h"
#include "epp_DomainCreate.h"
#include <domtools/dom_output.h>
#include <domtools/dom_wrapper.h>
#include "data/epp_domainXMLbase.h"
#include "data/epp_XMLException.h"


using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

void epp_DomainCreate::setRequestData(const epp_DomainCreateReq & parms)
{
   m_request.ref(new epp_DomainCreateReq(parms));
}

epp_DomainCreateRsp_ref epp_DomainCreate::getResponseData() const
{
   return m_response;
}


epp_string epp_DomainCreate::toXML()
{
   if(m_request == NULL || m_request->m_cmd == NULL || m_request->m_name == NULL) {
      throw epp_XMLException(__FILE__, __LINE__, "Missing data in request");
   }
   
   EPP_output xml_request;
   xml_request.setWhitespace(Whitespace);
   
   createStandardXMLHeader(xml_request);
   
   xml_request.beginTag("command");
   
   xml_request.beginTag("create");
   xml_request.beginTag("domain:create");

   setCommandSchemaAttributes(xml_request);
   
   xml_request.putTag("domain:name",*m_request->m_name);
   
   if(m_request->m_period != NULL) {
      
      xml_request.beginTag("domain:period");
      
      if(m_request->m_period->m_unit != NULL) {
	 xml_request.putAttribute("unit", returnPeriodUnitType(*m_request->m_period->m_unit));
      }
      
      if(m_request->m_period->m_value != NULL) {
	 xml_request.putCDATA(ltoa(*m_request->m_period->m_value));
      }
      xml_request.endTag("domain:period");
   } // if "m_period"
   
   if(m_request->m_name_servers != NULL) {
      
       xml_request.beginTag("domain:ns");
      for(epp_string_seq::iterator i = m_request->m_name_servers->begin();
	  i != m_request->m_name_servers->end(); i++)
	{
	   xml_request.putTag("domain:hostObj", *i);
	}
       xml_request.endTag("domain:ns");
   } // if "m_name_servers"
   
   if(m_request->m_registrant != NULL) {
      xml_request.putTag("domain:registrant", *m_request->m_registrant);
   } // if "m_registrant"
   
   if(m_request->m_contacts != NULL) {
      
      for(epp_domain_contact_seq::iterator i = m_request->m_contacts->begin();
	  i != m_request->m_contacts->end(); i++)
	{
	   xml_request.beginTag("domain:contact");
	   
	   if(i->m_type != NULL) {
	      xml_request.putAttribute("type",returnContactType(*i->m_type));
	   }
	   if(i->m_id != NULL) {
	      xml_request.putCDATA(*i->m_id);
	   }
	   xml_request.endTag("domain:contact");
	   
	} // for
      
   } // if "m_contacts"
   
   if(m_request->m_auth_info != NULL) {
      addAuthInfoElement(xml_request, m_request->m_auth_info, "domain:authInfo");
   } // if "m_auth_info"
   

   xml_request.endTag("domain:create");   
   xml_request.endTag("create");
   
   addExtensionElements(xml_request, m_request->m_cmd->m_extentions);
   
   if(m_request->m_cmd->m_client_trid != NULL) {
      xml_request.putTag("clTRID",*m_request->m_cmd->m_client_trid);
   } // if "m_cmd->m_client_trid"
   
   xml_request.endTag("command");
   
   createStandardXMLFooter(xml_request);
   
   return xml_request.getString();  
}

void epp_DomainCreate::fromXML(const eppobject::epp::epp_string & xml)
{
   dom_ptr doc = createDOMDocument(xml);
   
   dom_ptr responseNode = dom_ptr(doc->getTag("response"));
   
   if(responseNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find response-element");
   }
   
   m_response.ref(new epp_DomainCreateRsp());
   
   epp_Response_ref res = createGenericResponse(responseNode);
   m_response->m_rsp = res;
   
   dom_ptr creDataNode = dom_ptr(getresDataInfo(responseNode, "creData"));

   if(creDataNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find creData-element");
   }
   
   dom_ptr creChildNode = dom_ptr(creDataNode->getFirstChildElement());
   
   string elemPrefix = creDataNode->getPrefix();
   
   while (!creChildNode->isNull()) {
      
      string nodeName = creChildNode->getNodeName();
      
      if(nodeName == string(elemPrefix + ":name")) {
	 m_response->m_name.ref(new epp_string(creChildNode->getNodeData()));
//	 cout << "name: " << *m_response->m_name << endl;
      } // if "name"
      
      else if(nodeName == string(elemPrefix + ":crDate")) {
	 m_response->m_creation_date.ref(new epp_datetime(creChildNode->getNodeData()));
//	 cout << "crDate: " << *m_response->m_creation_date << endl;
      }
      
      else if(nodeName == string(elemPrefix + ":exDate")) {
	 m_response->m_expiration_date.ref(new epp_datetime(creChildNode->getNodeData()));
//	 cout << "exDate: " << *m_response->m_expiration_date << endl;
      } // if "exDate"
      
      creChildNode = dom_ptr(creChildNode->getNextSiblingElement());
   } // while "creChildNode"
   
}
