/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#include "config.h"
#include "epp_DomainUpdate.h"
#include <domtools/dom_output.h>
#include <domtools/dom_wrapper.h>
#include "data/epp_domainXMLbase.h"
#include "data/epp_XMLException.h"


using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

void epp_DomainUpdate::setRequestData(const epp_DomainUpdateReq & parms)
{
   m_request.ref(new epp_DomainUpdateReq(parms));
}

epp_DomainUpdateRsp_ref epp_DomainUpdate::getResponseData() const
{
   return m_response;
}


epp_string epp_DomainUpdate::toXML()
{
   if(m_request == NULL || m_request->m_cmd == NULL || m_request->m_name == NULL) {
      throw epp_XMLException(__FILE__, __LINE__, "Missing data in request");
   }
   
   EPP_output xml_request;
   xml_request.setWhitespace(Whitespace);

   createStandardXMLHeader(xml_request);
   
   xml_request.beginTag("command");
   
   xml_request.beginTag("update");
   xml_request.beginTag("domain:update");
   
   setCommandSchemaAttributes(xml_request);
   
   xml_request.putTag("domain:name",*m_request->m_name);
   
   addRemAddElement(xml_request, m_request->m_add, "add");
   addRemAddElement(xml_request, m_request->m_remove, "rem");
   addChangeElement(xml_request, m_request->m_change);

   xml_request.endTag("domain:update");   
   xml_request.endTag("update");
   
   addExtensionElements(xml_request, m_request->m_cmd->m_extentions);
   
   if(m_request->m_cmd->m_client_trid != NULL) {
      xml_request.putTag("clTRID",*m_request->m_cmd->m_client_trid);
   } // if "m_cmd->m_client_trid"
   
   xml_request.endTag("command");
   
   createStandardXMLFooter(xml_request);
   
   return xml_request.getString();  
}

void epp_DomainUpdate::fromXML(const eppobject::epp::epp_string & xml)
{
   dom_ptr doc = createDOMDocument(xml);
   
   dom_ptr responseNode = dom_ptr(doc->getTag("response"));
   
   if(responseNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find response-element");
   }
   
   epp_Response_ref res = createGenericResponse(responseNode);
   
   m_response.ref(new epp_DomainUpdateRsp(res));
   
}


void epp_DomainUpdate::addRemAddElement(eppobject::epp::EPP_output & outputobject,
					const epp_DomainUpdateAddRemove_ref & element,
					const eppobject::epp::epp_string & mode) const
{
   if(element == NULL) {
      return;
   }

   outputobject.beginTag("domain:" + mode);
   
   if(element->m_name_servers != NULL) {      
       outputobject.beginTag("domain:ns");
      for(epp_string_seq::iterator i = element->m_name_servers->begin();
	  i != element->m_name_servers->end(); i++)
	{
	   outputobject.putTag("domain:hostObj", *i);
	}
       outputobject.endTag("domain:ns");
   } // if "m_name_servers"
   
   if(element->m_contacts != NULL) {
      for(epp_domain_contact_seq::iterator i = element->m_contacts->begin();
	  i != element->m_contacts->end(); i++)
	{
	   outputobject.beginTag("domain:contact");
	   if(i->m_type != NULL) {
	      outputobject.putAttribute("type",returnContactType(*i->m_type));
	   }
	   if(i->m_id != NULL) {
	      outputobject.putCDATA(*i->m_id);
	   }
	   outputobject.endTag("domain:contact");
	}
   } // if "m_contacts"
   
   if(element->m_status != NULL) {
      for(epp_domain_status_seq::iterator i = element->m_status->begin();
	  i != element->m_status->end(); i++)
	{
	   if(i->m_type == NULL) {
	      throw epp_XMLException(__FILE__, __LINE__, "Missing data in domainstatus-list");
	   }
	   
	   outputobject.beginTag("domain:status");
	   outputobject.putAttribute("s",returnStatusType(*i->m_type));
	   
	   if(i->m_lang != NULL) {
	      outputobject.putAttribute("lang",*i->m_lang);
	   }
	   if(i->m_value != NULL) {
	      outputobject.putCDATA(*i->m_value);
	   }
	   outputobject.endTag("domain:status");
	}
   } // if "m_status"
   
   outputobject.endTag("domain:" + mode);
}

void epp_DomainUpdate::addChangeElement(eppobject::epp::EPP_output & outputobject,
					const epp_DomainUpdateChange_ref & element) const
{
   if(element == NULL) {
      return;
   }
   
   outputobject.beginTag("domain:chg");
   
   if(element->m_registrant != NULL) {
      outputobject.putTag("domain:registrant", *element->m_registrant);
   } // if "m_registrant"
   
   if(element->m_auth_info != NULL) {
      addAuthInfoElement(outputobject, element->m_auth_info, "domain:authInfo");
   } // if "m_auth_info"
   
   outputobject.endTag("domain:chg");
}

