/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_EXTENSION_H
#define __EPP_EXTENSION_H

/** @file epp_Extension.h
 *  @brief Definition of epp_Extension and epp_Extension_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "data/epp_eppdata.h"

namespace eppobject { namespace epp {

   /// Extension element used for server-defined response extensions.
   class epp_Extension {
      
    public:
      
      /// Constructor
      epp_Extension() {};

      /// Deconstructor
      virtual ~epp_Extension() {};
      
      /** Abstract function used to parse an epp object, and then return the
       *  corresponding correctly formatted EPP xml as an #epp_string
       * @return An empty #epp_string
       */
      virtual eppobject::epp::epp_string toXML() { return eppobject::epp::epp_string(); };
      
      /** Abstract function used to parse an #epp_string, and put the elements into an
       *  epp object. To be overloaded by sub-classes
       * @param xml An XML #epp_string
       */
      virtual void fromXML(const eppobject::epp::epp_string &) {};
   };
   
   /// Typedef for the ref to the epp_Extension class
   typedef refcnt_ptr<epp_Extension> epp_Extension_ref;
   
}}

#endif
