/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#include "config.h"
#include "epp_HostCheck.h"
#include <domtools/dom_output.h>
#include <domtools/dom_wrapper.h>
#include "data/epp_hostXMLbase.h"
#include "data/epp_XMLException.h"


using namespace std;
using namespace domtools;
using namespace eppobject::host;
using namespace eppobject::epp;

void epp_HostCheck::setRequestData(const epp_HostCheckReq & parms)
{
   m_request.ref(new epp_HostCheckReq(parms));
}

epp_HostCheckRsp_ref epp_HostCheck::getResponseData() const
{
   return m_response;
}


epp_string epp_HostCheck::toXML()
{
   if(m_request == NULL || m_request->m_cmd == NULL || m_request->m_names == NULL) {
      throw epp_XMLException(__FILE__, __LINE__, "Missing data in request");
   }
   
   EPP_output xml_request;
   xml_request.setWhitespace(Whitespace);
   
   createStandardXMLHeader(xml_request);
   
   xml_request.beginTag("command");
   
   xml_request.beginTag("check");

   xml_request.beginTag("host:check");

   setCommandSchemaAttributes(xml_request);
   
   for(epp_string_seq::iterator i = m_request->m_names->begin();
       i != m_request->m_names->end(); i++)
     {
	xml_request.putTag("host:name",*i);
     }
   
   xml_request.endTag("host:check");
   
   xml_request.endTag("check");
   
   addExtensionElements(xml_request, m_request->m_cmd->m_extentions);
   
   if(m_request->m_cmd->m_client_trid != NULL) {
      xml_request.putTag("clTRID",*m_request->m_cmd->m_client_trid);
   } // if "m_cmd->m_client_trid"
   
   xml_request.endTag("command");
   
   createStandardXMLFooter(xml_request);
   
   return xml_request.getString();  
}

void epp_HostCheck::fromXML(const eppobject::epp::epp_string & xml)
{
   dom_ptr doc = createDOMDocument(xml);
   
   dom_ptr responseNode = dom_ptr(doc->getTag("response"));
   
   if(responseNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find response-element");
   }
   
   epp_Response_ref res = createGenericResponse(responseNode);
   
   m_response.ref(new epp_HostCheckRsp());
   m_response->m_rsp = res;
   
   dom_ptr chkDataNode = dom_ptr(getresDataInfo(responseNode, "chkData"));
   
   if(chkDataNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find chkData-element");
   }
   
   dom_ptr chkChildNode = dom_ptr(chkDataNode->getFirstChildElement());
   
   string elemPrefix = chkChildNode->getPrefix();
   
   while(!chkChildNode->isNull()) {
      
      if(chkChildNode->getNodeName() == string(elemPrefix + ":cd")) {
	 
	 if(m_response->m_results == NULL) {
	    m_response->m_results.ref(new epp_check_result_seq);
	 }
	 
	 epp_CheckResult chRes;
	 
	 dom_ptr chkResChildNode = dom_ptr(chkChildNode->getFirstChildElement());
	 
	 while(!chkResChildNode->isNull()) {
	    string nodeName = chkResChildNode->getNodeName();
	    
	    if(nodeName == string(elemPrefix + ":name")) {
	       string avail = chkResChildNode->getAttribute("avail");
//	       cout << "avail: " << avail << endl;
	       
	       //lint -e1556
	       // We know that the new-assignments are not really suspicious...
	       if(avail == "1" || avail == "true") chRes.m_avail.ref(new epp_boolean(true));
	       else if(avail == "0" || avail == "false") chRes.m_avail.ref(new epp_boolean(false));
	       else throw epp_XMLException(__FILE__, __LINE__, "Empty or unknown avail-attribute");
	       //lint +e1556
		       
	       chRes.m_value.ref(new epp_string(chkResChildNode->getNodeData()));
//	       cout << "value: " << *chRes.m_value << endl;
	    } // if "name"
		  
	    else if(nodeName == string(elemPrefix + ":reason")) {
	       
	       string lang = chkResChildNode->getAttribute("lang");
	       if(!lang.empty()) chRes.m_lang.ref(new epp_string(lang));
		       
	       chRes.m_reason.ref(new epp_string(chkResChildNode->getNodeData()));
//	       cout << "reason: " << *chRes.m_reason << endl;
	    } // if "reason"
	    
	    chkResChildNode = dom_ptr(chkResChildNode->getNextSiblingElement());
	 } // while "chkResChildNode"
	 
	 m_response->m_results->push_back(chRes);
      } // if "cd"
      
      chkChildNode = dom_ptr(chkChildNode->getNextSiblingElement());
   } // while "chkChildNode"
   
}
