/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#include "config.h"
#include "epp_HostInfo.h"
#include <domtools/dom_output.h>
#include <domtools/dom_wrapper.h>
#include "data/epp_hostXMLbase.h"
#include "data/epp_XMLException.h"


using namespace std;
using namespace domtools;
using namespace eppobject::host;
using namespace eppobject::epp;

void epp_HostInfo::setRequestData(const epp_HostInfoReq & parms)
{
   m_request.ref(new epp_HostInfoReq(parms));
}

epp_HostInfoRsp_ref epp_HostInfo::getResponseData() const
{
   return m_response;
}


epp_string epp_HostInfo::toXML()
{
   if(m_request == NULL || m_request->m_cmd == NULL || m_request->m_name == NULL) {
      throw epp_XMLException(__FILE__, __LINE__, "Missing data in request");
   }
   
   EPP_output xml_request;
   xml_request.setWhitespace(Whitespace);
   
   createStandardXMLHeader(xml_request);
   
   xml_request.beginTag("command");
   
   xml_request.beginTag("info");
   xml_request.beginTag("host:info");

   setCommandSchemaAttributes(xml_request);
   
   xml_request.putTag("host:name",*m_request->m_name);
   
   xml_request.endTag("host:info");   
   xml_request.endTag("info");
   
   addExtensionElements(xml_request, m_request->m_cmd->m_extentions);
   
   if(m_request->m_cmd->m_client_trid != NULL) {
      xml_request.putTag("clTRID",*m_request->m_cmd->m_client_trid);
   } // if "m_cmd->m_client_trid"
   
   xml_request.endTag("command");
   
   createStandardXMLFooter(xml_request);
   
   return xml_request.getString();  
}


void epp_HostInfo::fromXML(const eppobject::epp::epp_string & xml)
{
   dom_ptr doc = createDOMDocument(xml);
   
   dom_ptr responseNode = dom_ptr(doc->getTag("response"));
   
   if(responseNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find response-element");
   }
   
   m_response.ref(new epp_HostInfoRsp());
   
   epp_Response_ref res = createGenericResponse(responseNode);
   m_response->m_rsp = res;
   
   dom_ptr infDataNode = dom_ptr(getresDataInfo(responseNode, "infData"));
   
   if(infDataNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find infData-element");
   }
   
   dom_ptr infChildNode = dom_ptr(infDataNode->getFirstChildElement());
   
   if(infChildNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Empty infData-element");
   }
   
   string elemPrefix = infDataNode->getPrefix();
   
   while(!infChildNode->isNull()) {
      
      string nodeName = infChildNode->getNodeName();
      
      if(nodeName == string(elemPrefix + ":name")) {
	 m_response->m_name.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "name: " << *m_response->m_name << endl;
      } // if "name"
      
      else if(nodeName == string(elemPrefix + ":roid")) {
	 m_response->m_roid.ref(new epp_roid(infChildNode->getNodeData()));
//	 cout << "roid: " << *m_response->m_roid << endl;
      } // if "roid"
      
      else if(nodeName == string(elemPrefix + ":status")) {
	 
	 if(m_response->m_status == NULL) {
	    m_response->m_status.ref(new epp_host_status_seq);
	 }
	 
	 epp_HostStatus hStatus;
	 
	 string type = infChildNode->getAttribute("s");
	 if(!type.empty()) {
	    hStatus.m_type.ref(new epp_HostStatusType(returnStatusEnumType(type)));
	 }
	 
	 string lang = infChildNode->getAttribute("lang");
	 if(!lang.empty()) {
	    hStatus.m_lang.ref(new epp_string(lang));
	 }
	 
	 string value = infChildNode->getNodeData();
	 if(!value.empty()) {
	    hStatus.m_value.ref(new epp_string(value));
	 }
	 
	 m_response->m_status->push_back(hStatus);
//	 cout << "status: " << type << ", " << lang << ", " << value <<  endl;
      } // if "status"
      
      else if(nodeName == string(elemPrefix + ":addr")) {
	 
	 if(m_response->m_addresses == NULL) {
	    m_response->m_addresses.ref(new epp_host_address_seq);
	 }
	 
	 epp_HostAddress hAddress;
	 
	 string type = infChildNode->getAttribute("ip");
	 if(!type.empty()) {
	    hAddress.m_type.ref(new epp_HostAddressType(returnIPTypeEnum(type)));
	 }
	 
	 string ip = infChildNode->getNodeData();
	 if(!ip.empty()) {
	    hAddress.m_ip.ref(new epp_string(ip));
	 }
	 
	 m_response->m_addresses->push_back(hAddress);
//	 cout << "addr: " << type << ", " << ip << endl;
      } // if "addr"
      
      else if(nodeName == string(elemPrefix + ":clID")) {
	 m_response->m_client_id.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "clID: " << *m_response->m_client_id << endl;
      } // if "clID"
      
      else if(nodeName == string(elemPrefix + ":crID")) {
	 m_response->m_created_by.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "crID: " << *m_response->m_created_by << endl;
      } // if "crID"
      
      else if(nodeName == string(elemPrefix + ":crDate")) {
	 m_response->m_created_date.ref(new epp_datetime(infChildNode->getNodeData()));
//	 cout << "crDate: " << *m_response->m_created_date << endl;
      } // if "crDate"
      
      else if(nodeName == string(elemPrefix + ":upID")) {
	 m_response->m_updated_by.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "upID: " << *m_response->m_updated_by << endl;
      } // if "upID"
      
      else if(nodeName == string(elemPrefix + ":upDate")) {
	 m_response->m_updated_date.ref(new epp_datetime(infChildNode->getNodeData()));
//	 cout << "upDate: " << *m_response->m_updated_date << endl;
      } // if "upDate"
      
      else if(nodeName == string(elemPrefix + ":trDate")) {
	 m_response->m_transfer_date.ref(new epp_datetime(infChildNode->getNodeData()));
//	 cout << "trDate: " << *m_response->m_transfer_date << endl;
      } // if "trDate"
      
      infChildNode = dom_ptr(infChildNode->getNextSiblingElement());
   } // while "infChildNode"
   
   
}
