/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#include "config.h"
#include "epp_Login.h"
#include <domtools/dom_output.h>
#include <domtools/dom_wrapper.h>
#include "data/epp_eppXMLbase.h"
#include "data/epp_XMLException.h"


using namespace domtools;
using namespace eppobject::epp;

void epp_Login::setRequestData(const epp_LoginReq & data)
{
   m_request.ref(new epp_LoginReq(data));
}

epp_LoginRsp_ref epp_Login::getResponseData() const
{
   return m_response;
}

epp_string epp_Login::toXML()
{
   if(m_request == NULL || m_request->m_cmd == NULL) {
      throw epp_XMLException(__FILE__, __LINE__, "Missing data in request");
   }
   
   EPP_output xml_request;
   xml_request.setWhitespace(Whitespace);
   
   createStandardXMLHeader(xml_request);
   
   xml_request.beginTag("command");
   
   xml_request.beginTag("login");
   
   if(m_request->m_client_id != NULL) {
      xml_request.putTag("clID", *m_request->m_client_id);
   } // if "m_client_id"
   
   if(m_request->m_password != NULL) {
      xml_request.putTag("pw", *m_request->m_password);
   } // if "m_password"
   
   if(m_request->m_new_password != NULL) {
      xml_request.putTag("newPW", *m_request->m_new_password);
   } // if "m_new_password"
   
   if(m_request->m_options != NULL) {
      xml_request.beginTag("options");
      
      if(m_request->m_options->m_version != NULL) {
	 xml_request.putTag("version", *m_request->m_options->m_version);
      }
      if(m_request->m_options->m_lang != NULL) {
	 xml_request.putTag("lang", *m_request->m_options->m_lang);
      }
      xml_request.endTag("options");
   } // if "m_options"

   xml_request.beginTag("svcs");
   
   if(m_request->m_services != NULL) {
      
      for(epp_string_seq::iterator i = m_request->m_services->begin();
	  i != m_request->m_services->end(); i++)
	{
	   xml_request.putTag("objURI", *i);
	}
   } // if "m_services"
   
   if(m_request->m_extensions != NULL) {
      
      xml_request.beginTag("svcExtension");
      
      for(epp_string_seq::iterator i = m_request->m_extensions->begin();
	  i != m_request->m_extensions->end(); i++)
	{
	   xml_request.putTag("extURI", *i);
	}
      
      xml_request.endTag("svcExtension");
   } // if "m_extensions"
   
   xml_request.endTag("svcs");
   
   xml_request.endTag("login");
   
   addExtensionElements(xml_request, m_request->m_cmd->m_extentions);
   
   if(m_request->m_cmd->m_client_trid != NULL) {
      xml_request.putTag("clTRID",*m_request->m_cmd->m_client_trid);
   } // if "m_cmd->m_client_trid"
   
   xml_request.endTag("command");
   
   createStandardXMLFooter(xml_request);
   
   return xml_request.getString();
   
}


void epp_Login::fromXML(const eppobject::epp::epp_string & xml)
{
   dom_ptr doc = createDOMDocument(xml);
   
   dom_ptr responseNode = dom_ptr(doc->getTag("response"));
   
   if(responseNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find response-element");
   }
   
   epp_Response_ref res = createGenericResponse(responseNode);
   
   m_response.ref(new epp_LoginRsp(res));
}
