/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/main_domaininfo.cc,v 1.17 2004/12/09 18:41:23 ewang2004 Exp $ */

#include <string>
#include <iostream>

#include "data/epp_DomainInfoReq.h"
#include "data/epp_DomainInfoRsp.h"
#include "data/epp_domainXMLbase.h"
#include "data/epp_XMLException.h"
#include "data/epp_Exception.h"
#include "epp_Action.h"
#include "epp_DomainInfo.h"
#include "epp_Session.h"
#include "testTools.h"

using namespace std;
using namespace eppobject::domain;
//using namespace eppobject::epp; //<fixme>to solve "ALL" conflict...</fixme>


void printRsp(const epp_DomainInfoRsp_ref & req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {
      
      printResponsePart1(req->m_rsp);
      printResDataHeaders("domain", "infData");
      
      if(req->m_name != NULL)
        cerr << "<domain:name>" << *req-> m_name << "</domain:name>" << endl;
      
      if(req->m_roid != NULL)
	cerr << "<domain:roid>" << *req->m_roid << "</domain:roid>" << endl;
            
      if(req->m_status != NULL)
	{
	   for(epp_domain_status_seq::iterator i = req->m_status->begin();
	       i != req->m_status->end(); i++)
	     {
		cerr << "<domain:status";
		if(i->m_type != NULL) {
		   string status = returnStatusType(*i->m_type);
		   cerr << " s=\"" << status << "\"";
		   
		}
		if(i->m_lang != NULL)
		  cerr << " lang=\"" << *i->m_lang << "\"";
		if(i->m_value != NULL) {
		   cerr << ">"<<*i->m_value<<"</domain:status>"<<endl;
		} else {
		   cerr << "/>" << endl;
                }
	     }
	}

      if(req->m_registrant != NULL)
	cerr << "<domain:registrant>" << *req->m_registrant << "</domain:registrant>" << endl;
      

      if(req->m_contacts != NULL)
	{
	   for(epp_domain_contact_seq::iterator i = req->m_contacts->begin();
	       i != req->m_contacts->end(); i++)
	     {
		cerr << "<domain:contact";
		if(i->m_type != NULL) {
		   string type = returnContactType(*i->m_type);
		   cerr << " type=\"" << type << "\"";
		   
		}
		cerr << ">";
		
		if(i->m_id != NULL)
		  cerr << *i->m_id;
		
		cerr << "</domain:contact>" << endl;
	     }
	}
      
      if(req->m_name_servers != NULL)
	{
           cerr << "<domain:ns>" << endl;
	   for(eppobject::epp::epp_string_seq::iterator i = req->m_name_servers->begin();
	       i != req->m_name_servers->end(); i++)
	     {
	       cerr << "<domain:hostObj>" << *i << "</domain:hostObj>" << endl;
	     }
           cerr << "</domain:ns>" << endl;
	}
      
      if(req->m_hosts != NULL)
	{
	   for(eppobject::epp::epp_string_seq::iterator i = req->m_hosts->begin();
	       i != req->m_hosts->end(); i++)
	     {
                cerr << "<domain:host>" << *i << "</domain:host>" << endl;
	     }
	}
      
      if(req->m_client_id != NULL)
	cerr << "<domain:clID>" << *req->m_client_id << "</domain:clID>" << endl;
      
      if(req->m_created_by != NULL)
	cerr << "<domain:crID>" << *req->m_created_by << "</domain:crID>" << endl;
      
      if(req->m_created_date != NULL)
	cerr << "<domain:crDate>" << *req->m_created_date << "</domain:crDate>" << endl;
      
      if(req->m_updated_by != NULL)
	cerr << "<domain:upID>" << *req->m_updated_by << "</domain:upID>" << endl;
      
      if(req->m_updated_date != NULL)
	cerr << "<domain:upDate>" << *req->m_updated_date << "</domain:upDate>" << endl;
      
      if(req->m_expiration_date != NULL)
	cerr << "<domain:exDate>" << *req->m_expiration_date << "</domain:exDate>" << endl;      
      
      if(req->m_transfer_date != NULL)
	cerr << "<domain:trDate>" << *req->m_transfer_date << "</domain:trDate>" << endl;      
      
      if(req->m_auth_info != NULL) {
	 cerr << "<domain:authInfo";
	 
	 if(req->m_auth_info->m_roid != NULL)
	    cerr << " roid=\"" << *req->m_auth_info->m_roid << "\"";
	 
	 cerr << ">";
	 
	 if(req->m_auth_info->m_value != NULL) {

            string type;

	    if(req->m_auth_info->m_type != NULL) {
	       switch(*req->m_auth_info->m_type) {
	        case eppobject::epp::PW: type = "pw"; break;
	        case eppobject::epp::EXT: type = "ext"; break;
	        default: type = "**UNKNOWN**";
	       }
	    }	 

	    cerr << "\n\t<domain:"<<type<<">"<<*req->m_auth_info->m_value<<"</domain:"<<type<<">\n";
         }
	 
	 cerr << "</domain/:authInfo>" << endl;
	 
      }
      
      printResDataFooter("domain", "infData");
      printResponsePart2(req->m_rsp);
   } // if(req != NULL)
   
   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {
   
   eppobject::epp::epp_Session test;
   
   string username, password;
   
   string extra_usage = "[-extradata=trid=xxx:domain=abc.tld";
   if(!handleArgs(test, argc, argv, username, password, extra_usage))
     return 1;

   map<string,string> extraDataMap;
   
   if(argc > 1) {
      for(int parm = 1; parm < argc; parm++) {
	 
	 if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
            cout << "Found extra data param..." << endl;
#endif
	    string td = &argv[parm][11];
	    extraDataMap = getExtraDataMap(td);
	 }
      }
   }
   
   testLogIn(test, username, password);
   
   try {
      
      epp_DomainInfoReq_ref l_req(new epp_DomainInfoReq());

      string trID = "ABC-12345";
      if(extraDataMap.find("trid") != extraDataMap.end()) trID = extraDataMap["trid"];
      
      l_req->m_cmd.ref(new eppobject::epp::epp_Command(NULL,
						       NULL,
						       eppobject::epp::epp_trid(trID)));
      
      if(extraDataMap.find("domain") == extraDataMap.end()) {
	 l_req->m_name.ref(new eppobject::epp::epp_string("example.com"));
      }
      else {
	 l_req->m_name.ref(new eppobject::epp::epp_string(extraDataMap["domain"]));
      }
      
      l_req->m_hosts_type.ref(new epp_DomainHostsType(ALL));
      
      epp_DomainInfo_ref l_obj(new epp_DomainInfo());
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      epp_DomainInfoRsp_ref l_rsp = l_obj->getResponseData();
      
#ifndef TEST_IO_REQUEST2CERR
      printRsp(l_rsp);
#endif
      return 0;
      
   }
   catch(const eppobject::epp::epp_XMLException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(const eppobject::epp::epp_Exception & ex) {
      cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
      cerr << "<response>" << endl;
      printResultsSeq(ex.m_details);
      printTransID(ex.m_trans_id);
      cerr << "</response>" << endl;
#endif
   }
   catch(const epptransport::epp_TrException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }
   
}
