/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/main_domainupdate.cc,v 1.19 2004/12/09 18:42:39 ewang2004 Exp $ */

#include <vector>
#include <iostream>

#include "data/epp_DomainUpdateReq.h"
#include "data/epp_DomainUpdateRsp.h"
#include "data/epp_XMLException.h"
#include "data/epp_Exception.h"
#include "data/epp_domainXMLbase.h"
#include "epp_Action.h"
#include "epp_DomainUpdate.h"
#include "epp_Session.h"
#include "testTools.h"

using namespace std;
using namespace eppobject::domain;
using namespace eppobject::epp;


void printRsp(const epp_DomainUpdateRsp_ref & req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL)
     printResponsePart(req->m_rsp);
   
   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {
   
   epp_Session test;
   
   string username, password;
   
   vector<string> extra_usage;
   extra_usage.push_back("[-extradata=trid=xxx:domain=abc");
   extra_usage.push_back(" -add=status1=aaa/reason{:...}:ns1=222{:...}:contact1=t/abc123{:...}");
   extra_usage.push_back(" -remove=status1=ccc/reason{:...}:ns1=111{:...}:contact1=a/abc987{:...}");
   extra_usage.push_back(" -change=authinfo=pw/123123]");
   extra_usage.push_back("\tIf you included spaces in the status reason, you need to quote the entire 'add' or 'remove' param");
   if(!handleArgs(test, argc, argv, username, password, extra_usage))
     return 1;

   map<string,string> extraDataMap;
   map<string,string> addDataMap;
   map<string,string> changeDataMap;
   map<string,string> removeDataMap;
   
   if(argc > 1) {
      for(int parm = 1; parm < argc; parm++) {
	 
	 if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
            cout << "Found extra data param..." << endl;
#endif
	    string td = &argv[parm][11];
	    extraDataMap = getExtraDataMap(td);
	 }

	 if (!strncmp(argv[parm], "-add=", 5)) {
#ifdef RTK_DEBUG
            cout << "Found 'add' data param..." << endl;
#endif
	    string td = &argv[parm][5];
	    addDataMap = getExtraDataMap(td);
            if ( addDataMap.empty() ) addDataMap[""] = "dummy";
	 }

	 if (!strncmp(argv[parm], "-change=", 8)) {
#ifdef RTK_DEBUG
            cout << "Found 'change' data param..." << endl;
#endif
	    string td = &argv[parm][9];
	    changeDataMap = getExtraDataMap(td);
            if ( changeDataMap.empty() ) changeDataMap[""] = "dummy";
	 }

	 if (!strncmp(argv[parm], "-remove=", 8)) {
#ifdef RTK_DEBUG
            cout << "Found 'remove' data param..." << endl;
#endif
	    string td = &argv[parm][8];
	    removeDataMap = getExtraDataMap(td);
            if ( removeDataMap.empty() ) removeDataMap[""] = "dummy";
	 }
      }
   }
   
   testLogIn(test, username, password);
   
   try {      
      
      epp_DomainUpdateReq_ref l_req(new epp_DomainUpdateReq());
      
      string trID = "ABC-12345";
      if(extraDataMap.find("trid") != extraDataMap.end()) trID = extraDataMap["trid"];
      
      l_req->m_cmd.ref(new epp_Command(NULL,
				       epp_trid(trID)));
      
      if( addDataMap.empty() &&
          removeDataMap.empty() &&
          changeDataMap.empty() ) {
	 
	 l_req->m_add.ref(new epp_DomainUpdateAddRemove());
	 l_req->m_add->m_name_servers.ref(new epp_string_seq());
	 l_req->m_add->m_name_servers->push_back("ns1.example2.com");
	 
	 l_req->m_add->m_contacts.ref(new epp_domain_contact_seq());
	 epp_DomainContact Cont1(TECH, epp_string("mak21"));
	 l_req->m_add->m_contacts->push_back(Cont1);
	 
	 l_req->m_add->m_status.ref(new epp_domain_status_seq());
	 epp_DomainStatus stat;
	 stat.m_type.ref(new epp_DomainStatusType(CLIENT_HOLD));
	 stat.m_lang.ref(new epp_string("en"));
	 stat.m_value.ref(new epp_string("Payment overdue."));
	 l_req->m_add->m_status->push_back(stat);
	 
	 l_req->m_remove.ref(new epp_DomainUpdateAddRemove());
	 l_req->m_remove->m_name_servers.ref(new epp_string_seq());
	 l_req->m_remove->m_name_servers->push_back("ns2.example.com");
	 l_req->m_remove->m_contacts.ref(new epp_domain_contact_seq());
	 epp_DomainContact Cont2(TECH, epp_string("sh8013"));
	 l_req->m_remove->m_contacts->push_back(Cont2);
	 l_req->m_remove->m_status.ref(new epp_domain_status_seq());
	 epp_DomainStatus stat2;
	 stat2.m_type.ref(new epp_DomainStatusType(CLIENT_UPDATE_PROHIBITED));
	 l_req->m_remove->m_status->push_back(stat2);
	 
	 
	 l_req->m_change.ref(new epp_DomainUpdateChange());
	 l_req->m_change->m_registrant.ref(new epp_string("sh8013"));
	 l_req->m_change->m_auth_info.ref(new epp_AuthInfo());
	 l_req->m_change->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
	 l_req->m_change->m_auth_info->m_value.ref(new epp_string("2BARfoo"));
      }
      else {

	 if( ! addDataMap.empty() ) {
	    l_req->m_add.ref(new epp_DomainUpdateAddRemove());
	    
	    if( ! addDataMap["status1"].empty() ) {
	       char status_name[10];
               int status_number = 1;
               sprintf(status_name, "status%d", status_number);

	       l_req->m_add->m_status.ref(new epp_domain_status_seq());
	       
	       while( ! addDataMap[status_name].empty() ) {
		  string tmp = addDataMap[status_name];
		  
		  epp_DomainStatus status;

                  string type, value;
                  unsigned int sep = tmp.find("/");
                  if ( sep != string::npos ) {
                     type  = tmp.substr(0,sep);
                     value = tmp.substr(sep+1);
                     cout << "type ["<<type<<"] value ["<<value<<"]\n";
		     status.m_type.ref(new epp_DomainStatusType(returnStatusEnumType(type)));
	             status.m_value.ref(new epp_string(value));
                  } else {
		     status.m_type.ref(new epp_DomainStatusType(returnStatusEnumType(tmp)));
                  }
		  l_req->m_add->m_status->push_back(status);


                  status_number++;
                  sprintf(status_name, "status%d", status_number);
	       }
	    }

	    if( ! addDataMap["ns1"].empty() ) {
	       char ns_name[5];
               int ns_number = 1;
               sprintf(ns_name, "ns%d", ns_number);

	       l_req->m_add->m_name_servers.ref(new epp_string_seq());
	       
	       while( ! addDataMap[ns_name].empty() ) {
		  l_req->m_add->m_name_servers->push_back(addDataMap[ns_name]);
                  ns_number++;
                  sprintf(ns_name, "ns%d", ns_number);
	       }
	    }

	    if( ! addDataMap["contact1"].empty() ) {
               char key[10]; // contactxx
               
               l_req->m_add->m_contacts.ref(new epp_domain_contact_seq());
	       
               for(unsigned int n = 1; n <= 10; n++) {
                  sprintf(key,"contact%d",n);
                  if ( addDataMap.find(key) == addDataMap.end() ) break;
	          epp_DomainContact contact;
                  string id = addDataMap[key];
                  string type, value;
                  if ( ! id.empty() ) {
                     // contact1=t/abc
                     // contact2=a/qwerty
                     // etc...
                     type = id.substr(0,1);
                     value = id.substr(2);
                     if ( type == "t" ) contact.m_type.ref(new epp_DomainContactType(TECH));
                     if ( type == "a" ) contact.m_type.ref(new epp_DomainContactType(ADMIN));
                     if ( type == "b" ) contact.m_type.ref(new epp_DomainContactType(BILLING));
                     contact.m_id.ref(new epp_string(value));
	             l_req->m_add->m_contacts->push_back(contact);
                  }
	       }
	    }

	 }

	 
	 if( ! removeDataMap.empty() ) {
	    l_req->m_remove.ref(new epp_DomainUpdateAddRemove());
	    
	    if( ! removeDataMap["status1"].empty() ) {
	       char status_name[10];
               int status_number = 1;
               sprintf(status_name, "status%d", status_number);

	       l_req->m_remove->m_status.ref(new epp_domain_status_seq());
	       
	       while( ! removeDataMap[status_name].empty() ) {
		  string tmp = removeDataMap[status_name];
		  
		  epp_DomainStatus status;
                  string type, value;
                  unsigned int sep = tmp.find("/");
                  if ( sep != string::npos ) {
                     type  = tmp.substr(0,sep);
                     value = tmp.substr(sep+1);
                     cout << "type ["<<type<<"] value ["<<value<<"]\n";
		     status.m_type.ref(new epp_DomainStatusType(returnStatusEnumType(type)));
	             status.m_value.ref(new epp_string(value));
                  } else {
		     status.m_type.ref(new epp_DomainStatusType(returnStatusEnumType(tmp)));
                  }
		  l_req->m_remove->m_status->push_back(status);

                  status_number++;
                  sprintf(status_name, "status%d", status_number);
	       }
	    }

	    if( ! removeDataMap["ns1"].empty() ) {
	       char ns_name[5];
               int ns_number = 1;
               sprintf(ns_name, "ns%d", ns_number);

	       l_req->m_remove->m_name_servers.ref(new epp_string_seq());
	       
	       while( ! removeDataMap[ns_name].empty() ) {
		  l_req->m_remove->m_name_servers->push_back(removeDataMap[ns_name]);
                  ns_number++;
                  sprintf(ns_name, "ns%d", ns_number);
	       }
	    }

	    if( ! removeDataMap["contact1"].empty() ) {
               char key[10]; // contactxx
               
               l_req->m_remove->m_contacts.ref(new epp_domain_contact_seq());
	       
               for(unsigned int n = 1; n <= 10; n++) {
                  sprintf(key,"contact%d",n);
                  if ( removeDataMap.find(key) == removeDataMap.end() ) break;
	          epp_DomainContact contact;
                  string id = removeDataMap[key];
                  string type, value;
                  if ( ! id.empty() ) {
                     // contact1=t/abc
                     // contact2=a/qwerty
                     // etc...
                     type = id.substr(0,1);
                     value = id.substr(2);
                     if ( type == "t" ) contact.m_type.ref(new epp_DomainContactType(TECH));
                     if ( type == "a" ) contact.m_type.ref(new epp_DomainContactType(ADMIN));
                     if ( type == "b" ) contact.m_type.ref(new epp_DomainContactType(BILLING));
                     contact.m_id.ref(new epp_string(value));
	             l_req->m_remove->m_contacts->push_back(contact);
                  }
	       }
	    }

	 }

	 
	 if( ! changeDataMap.empty() ) {
	    
	    epp_DomainUpdateChange_ref chg(new epp_DomainUpdateChange());
	    
	    if(changeDataMap.find("authinfo") != changeDataMap.end()) {	    
               string authinfo = changeDataMap["authinfo"];
               string type, value;
               unsigned int sep = authinfo.find("/");
               if ( sep != string::npos ) {
                  type = authinfo.substr(0,sep);
                  value = authinfo.substr(sep+1);
	          l_req->m_change->m_auth_info.ref(new epp_AuthInfo());
                  if ( type == "pw" )  l_req->m_change->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
                  if ( type == "ext" ) l_req->m_change->m_auth_info->m_type.ref(new epp_AuthInfoType(EXT));
	          l_req->m_change->m_auth_info->m_value.ref(new epp_string(value));
               }
	    }

	 }
	 
      }
      
      if ( extraDataMap.find("domain") == extraDataMap.end() ) l_req->m_name.ref(new epp_string("example.com"));
      else l_req->m_name.ref(new epp_string(extraDataMap["domain"]));
      
      epp_DomainUpdate_ref l_obj(new epp_DomainUpdate());
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      epp_DomainUpdateRsp_ref l_rsp = l_obj->getResponseData();
      
#ifndef TEST_IO_REQUEST2CERR
      printRsp(l_rsp);
#endif
      return 0;
   
   }
   catch(const epp_XMLException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(const epp_Exception & ex) {
      cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
      cerr << "<response>" << endl;
      printResultsSeq(ex.m_details);
      printTransID(ex.m_trans_id);
      cerr << "</response>" << endl;
#endif
   }
   catch(const epptransport::epp_TrException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }
   
}
