/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/main_hello.cc,v 1.16 2005/09/13 12:44:08 jamesfgould Exp $ */

/** @file main_hello.cc
 *  @brief Test program for the <hello> command
 *  @author Asbjorn Steira Mikkelsen
 */

#include <string>
#include <iostream>

#include "data/epp_Greeting.h"
#include "data/epp_XMLException.h"
#include "data/epp_Exception.h"
#include "transport/epp_TransportBase.h"
#include "epp_Action.h"
#include "epp_Hello.h"
#include "epp_Session.h"
#include "testTools.h"

using namespace std;
using namespace eppobject::epp;

/** Function that write the server response to cerr
 * @param req Greeting returned from the server
 */
void printRsp(const epp_Greeting_ref & req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) { 
      cerr << "<greeting>" << endl;

      if(req->m_server_id != NULL)
	cerr << "<svID>" << *req->m_server_id << "</svID>" << endl;
      
      if(req->m_server_date != NULL)
	cerr << "<svDate>" << *req->m_server_date << "</svDate>" << endl;
      
      if(req->m_svc_menu != NULL) {
	 cerr << "<svcMenu>" << endl;

	 if(req->m_svc_menu->m_versions != NULL) {
	    for(epp_string_seq::iterator i = req->m_svc_menu->m_versions->begin();
		i != req->m_svc_menu->m_versions->end(); i++) {
	       cerr << "<version>" << *i << "</version>" << endl;
	    }
	 } // if "m_versions"
	 
	 if(req->m_svc_menu->m_langs != NULL) {
	    for(epp_string_seq::iterator i = req->m_svc_menu->m_langs->begin();
		i != req->m_svc_menu->m_langs->end(); i++) {
	       cerr << "<lang>" << *i << "</lang>" << endl;
	    }
	 } // if "m_langs"
	 
	 if(req->m_svc_menu->m_services != NULL) {
	    for(epp_objuri_seq::iterator i = req->m_svc_menu->m_services->begin();
		i != req->m_svc_menu->m_services->end(); i++) {
	       cerr << "<objURI>" << *i << "</objURI>" << endl;
	    }
	 } // if "m_services"
	 
	 if(req->m_svc_menu->m_extensions != NULL) {
	    cerr << "<svcExtension>" << endl;
	    for(epp_exturi_seq::iterator i = req->m_svc_menu->m_extensions->begin();
		i != req->m_svc_menu->m_extensions->end(); i++) {
	       cerr << "<extURI>" << *i << "</extURI>" << endl;
	    }
	    cerr << "</svcExtension>" << endl;
	 } // if "m_extensions"
	 
	 cerr << "</svcMenu>" << endl;
      } // if "m_svc_menu"
      
      if(req->m_dcp != NULL) {
	 
	 cerr << "<dcp>" << endl;
	 
	 if(req->m_dcp->m_access != NULL)
	   {
	      string access;
	      switch(*req->m_dcp->m_access) {
		 
	       case ALL: access = "all"; break;
	       case NONE_ACCESS: access = "noneAccess"; break;
	       case NULL_ACCESS: access = "null"; break;
	       case OTHER: access = "other"; break;
	       case PERSONAL: access = "personal"; break;
	       case PERSONAL_AND_OTHER: access = "personalAndOther"; break;
	       default: access = "**UNKNOWN**";
	      }
	      
	      cerr << "<access><" << access << "/></access>" << endl;
	   } // if "m_access"
	 
	 if(req->m_dcp->m_statements != NULL)
	   {
	      for(epp_dcpStatement_seq::iterator j = req->m_dcp->m_statements->begin();
		  j != req->m_dcp->m_statements->end(); j++)
		{
		   cerr << "<statement>" << endl;
		   
		   if(j->m_purposes != NULL)
		     {
			cerr << "<purpose>";
			for(epp_dcpPurpose_seq::iterator i = j->m_purposes->begin();
			    i != j->m_purposes->end(); i++) {
			   
			   string purpose;
			   switch(*i) {
			    case ADMIN_PURPOSE: purpose = "admin"; break;
			    case CONTACT: purpose = "contact"; break;
			    case OTHER_PURPOSE: purpose = "other"; break;
			    case PROV: purpose = "prov"; break;
			    default: purpose = "**UNKNOWN**";
			   }
			   
			   cerr << "<" << purpose << "/>";
			} // for
			
			cerr << "</purpose>" << endl;
		     } // if "m_purposes"
		   if(j->m_recipients != NULL)
		     {
			cerr << "<recipient>";
			for(epp_dcpRecipient_seq::iterator i = j->m_recipients->begin();
			    i != j->m_recipients->end(); i++) {
			   
			   string recipient;
			   switch(*i->m_type) {
			    case OTHER_RECIPIENT: recipient = "other/"; break;
			    case OURS:
					// JG - Changed to handle NULL m_rec_desc since it is optional
					if ( i->m_rec_desc != NULL ) {
						recipient = "ours>"+*i->m_rec_desc+"</ours";
					}
					else {
						recipient = "ours/";
					}
					break;
			    case PUBLIK: recipient = "public/"; break;
			    case SAME: recipient = "same/"; break;
			    case UNRELATED: recipient = "unrelated/"; break;
			    default: recipient = "**UNKNOWN**";
			   }
			   
			   cerr << "<" << recipient << ">";
			} // for
			cerr << "</recipient>" << endl;
		     } // if "m_recipients"
		   if(j->m_retention != NULL)
		     {
			cerr << "<retention>";
			   
			string retention;
			switch(*j->m_retention) {
			 case BUSINESS: retention = "business"; break;
			 case INDEFINITE: retention = "indefinite"; break;
			 case LEGAL: retention = "legal"; break;
			 case NONE: retention = "none"; break;
			 case STATED: retention = "stated"; break;
			 default: retention = "**UNKNOWN**";
			}
			
			cerr << "<" << retention << "/>";
			cerr << "</retention>" << endl;
		     } // if "retentions"
		   
		   cerr << "</statement>" << endl;
		}
	   } // if "m_statement"

	 if(req->m_dcp->m_expiry != NULL)
	   {
	      string expiry, value;
	      switch(*req->m_dcp->m_expiry->m_type) {
		 
	       case ABSOLUTE: expiry = "absolute"; break;
	       case RELATIVE: expiry = "relative"; break;
	       default: expiry = "**UNKNOWN**";
	      }
	      
              value = *req->m_dcp->m_expiry->m_value;
	      cerr << "<expiry><"<<expiry<<"/>"<<value<<"</"<<expiry<<"></expiry>" << endl;
	   } // if "m_access"
	 
      	 cerr << "</dcp>" << endl;
      } // if "m_dcp"
   }

   cerr << "</greeting>" << endl;
   
   cout << "_______________________________\n";
} // printRsp


/// Main part of test program for the <hello> command
int main(int argc, char * argv[]) {
   
   // Initialize session object
   epp_Session test;   
   
   string username, password;
   
   // Handle arguments. This function might change the session object
   // and username/password
   if(!handleArgs(test, argc, argv, username, password))
     return 1;
   
   try {
      
      // Initialize <hello> command object
      epp_Hello_ref l_obj(new epp_Hello());

      // Connect to server and get the greeting. In this
      // test program we do not do anything further with
      // the returned greeting.
      epp_Greeting_ref myGr = test.connectAndGetGreeting();
      
      // As the <hello> request has no user-defined attributes
      // we are ready to process the command. processAction will
      // write the data to the server, wait for the response,
      // parse the response, and put the data back into the
      // session object.
      test.processAction(l_obj);

      // We then return the response data, for this command a
      // greeting
      epp_Greeting_ref l_rsp = l_obj->getResponseData();
      
#ifndef TEST_IO_REQUEST2CERR
      // We then use the above function to print the response
      // from the server
      printRsp(l_rsp);
      cout << l_rsp->m_dcp << endl;
#endif
      // Finished
      return 0;

   }
   catch(const epp_XMLException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(const epp_Exception & ex) {
      cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
      cerr << "<response>" << endl;
      printResultsSeq(ex.m_details);
      printTransID(ex.m_trans_id);
      cerr << "</response>" << endl;
#endif
   }
   catch(const epptransport::epp_TrException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }   
}
