/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/main_hostcreate.cc,v 1.16 2003/07/18 20:21:19 tubadanm Exp $ */

#include <vector>
#include <iostream>

#include "data/epp_HostCreateReq.h"
#include "data/epp_HostCreateRsp.h"
#include "data/epp_XMLException.h"
#include "data/epp_Exception.h"
#include "data/epp_hostXMLbase.h"
#include "epp_Action.h"
#include "epp_HostCreate.h"
#include "epp_Session.h"
#include "testTools.h"


using namespace std;
using namespace eppobject::host;
using namespace eppobject::epp;


void printRsp(const epp_HostCreateRsp_ref & req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {
      
      printResponsePart1(req->m_rsp);
      printResDataHeaders("host", "creData");
      
      if(req->m_name != NULL)
	cerr << "<host:name>" << *req-> m_name << "</host:name>" << endl;
      
      if(req->m_creation_date != NULL)
	cerr << "<host:crDate>" << *req-> m_creation_date << "</host:crDate>" << endl;
      
      printResDataFooter("host", "creData");
      printResponsePart2(req->m_rsp);
      
   }
  
   
   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {
   
   epp_Session test;
   
   string username, password;
   
   vector<string> extra_usage;
   extra_usage.push_back("[-extradata=trid=xxx~host=abc~ip1=v4/1.2.3.4~ip2=v6/1080:0:0:0:8:800:200C:417A{~...}]");
   extra_usage.push_back("Note the use of '~' to separate keys and value, unlike the ':' from other examples");
   if(!handleArgs(test, argc, argv, username, password, extra_usage))
     return 1;

   map<string,string> extraDataMap;
   
   if(argc > 1) {
      for(int parm = 1; parm < argc; parm++) {
	 
	 if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
            cout << "Found extra data param..." << endl;
#endif
	    string td = &argv[parm][11];
	    extraDataMap = getExtraDataMap(td);
	 }
      }
   }
   
   testLogIn(test, username, password);
   
   try {
      
      epp_HostCreateReq_ref l_req(new epp_HostCreateReq());
   
      string trID = "ABC-12345";
      if(extraDataMap.find("trid") != extraDataMap.end()) trID = extraDataMap["trid"];
      
      l_req->m_cmd.ref(new epp_Command(NULL,
				       NULL,
				       epp_trid(trID)));
      
      if ( extraDataMap.find("host") == extraDataMap.end() ) {
         l_req->m_name.ref(new epp_string("ns1.example.com"));

	 epp_HostAddress host1(IPV4, epp_string("192.1.2.3"));      
	 epp_HostAddress host2(IPV4, epp_string("198.1.2.3"));
	 epp_HostAddress host3(IPV6, epp_string("1080:0:0:0:8:800:200C:417A"));
	 epp_HostAddress host4(IPV6, epp_string("::FFFF:129.144.52.38"));
	 
	 epp_host_address_seq_ref ipadr(new epp_host_address_seq);
	 
	 ipadr->push_back(host1);
	 ipadr->push_back(host2);
	 ipadr->push_back(host3);
	 ipadr->push_back(host4);
	 
	 l_req->m_addresses = ipadr;

      } else {
         l_req->m_name.ref(new epp_string(extraDataMap["host"]));
      }

      if ( extraDataMap.find("ip1") != extraDataMap.end() ) {
	 
	 epp_host_address_seq_ref ipadr(new epp_host_address_seq);

         char key[5];
	 for(unsigned int n = 1; n <= 14; n++) {

            sprintf(key,"ip%d",n);
            if ( extraDataMap.find(key) == extraDataMap.end() ) break;
            string ipstring = extraDataMap[key];
            if ( ipstring == "-1" ) break;
	    unsigned int sep = ipstring.find("/");
	    if(sep != string::npos) {
	       string iptype = ipstring.substr(0,sep);
	       string ip = ipstring.substr(sep+1);
	       epp_HostAddress host(returnIPTypeEnum(epp_string(iptype)), epp_string(ip));
	       ipadr->push_back(host);
	    }
	 }

	 l_req->m_addresses = ipadr;
	 
      }
	 
      epp_HostCreate_ref l_obj(new epp_HostCreate());
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      epp_HostCreateRsp_ref l_rsp = l_obj->getResponseData();
      
#ifndef TEST_IO_REQUEST2CERR
      printRsp(l_rsp);
#endif
      return 0;
      
   }
   catch(const epp_XMLException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(const epp_Exception & ex) {
      cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
      cerr << "<response>" << endl;
      printResultsSeq(ex.m_details);
      printTransID(ex.m_trans_id);
      cerr << "</response>" << endl;
#endif
   }
   catch(const epptransport::epp_TrException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }
   
}
