/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/main_hostinfo.cc,v 1.15 2003/07/18 20:21:19 tubadanm Exp $ */

#include <string>
#include <iostream>

#include "data/epp_HostInfoReq.h"
#include "data/epp_HostInfoRsp.h"
#include "data/epp_hostXMLbase.h"
#include "data/epp_XMLException.h"
#include "data/epp_Exception.h"
#include "epp_Action.h"
#include "epp_HostInfo.h"
#include "epp_Session.h"
#include "testTools.h"

using namespace std;
using namespace eppobject::host;
using namespace eppobject::epp;


void printRsp(const epp_HostInfoRsp_ref & req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {
      
      printResponsePart1(req->m_rsp);
      printResDataHeaders("host", "infData");
      
      if(req->m_name != NULL)
	cerr << "<host:name>" << *req->m_name << "</host:name>" << endl;
      
      if(req->m_roid != NULL)
	cerr << "<host:roid>" << *req->m_roid << "</host:roid>" << endl;
      
      if(req->m_status != NULL)
	{
	   for(epp_host_status_seq::iterator i = req->m_status->begin();
	       i != req->m_status->end(); i++)
	     {
		cerr << "<host:status";
		if(i->m_type != NULL) {
		   string status = returnStatusType(*i->m_type);
		   cerr << " s=\"" << status << "\"";
		}
		if(i->m_lang != NULL)
		  cerr << " m_lang :" << *i->m_lang << endl;
		if(i->m_value != NULL)
		  cerr << " m_value: " << *i->m_value << endl;
		
		cerr << "/>" << endl;
	     }
	}

      if(req->m_addresses != NULL)
	{
	   for(epp_host_address_seq::iterator i = req->m_addresses->begin();
	       i != req->m_addresses->end(); i++)
	     {
		cerr << "<host:addr";
		if(i->m_type != NULL) {
		   string status = returnIPTypeString(*i->m_type);
		   cerr << " ip=\"" << status << "\"";

		}
		cerr << ">";
		if(i->m_ip != NULL)
		  cerr << *i->m_ip;
		cerr << "</host:addr>" << endl;
	     }
	}

      if(req->m_client_id != NULL)
	cerr << "<host:clID>" << *req->m_client_id << "</host:clID>" << endl;
      
      if(req->m_created_by != NULL)
	cerr << "<host:crID>" << *req->m_created_by << "</host:crID>" << endl;
      
      if(req->m_created_date != NULL)
	cerr << "<host:crDate>" << *req->m_created_date << "</host:crDate>" << endl;
      
      if(req->m_updated_by != NULL)
	cerr << "<host:upID>" << *req->m_updated_by << "</host:upID>" << endl;
      
      if(req->m_updated_date != NULL)
	cerr << "<host:upDate>" << *req->m_updated_date << "</host:upDate>" << endl;
      
      if(req->m_transfer_date != NULL)
	cerr << "<host:trDate>" << *req->m_transfer_date << "</host:trDate>" << endl;

      printResDataFooter("host", "infData");
      printResponsePart2(req->m_rsp);

   }
   
   
   cout << "_______________________________\n";
} // printRsp



int main(int argc, char * argv[]) {   
   
   epp_Session test;
   
   string username, password;
   
   string extra_usage = "[-extradata=trid=xxx:host=abc]";
   if(!handleArgs(test, argc, argv, username, password, extra_usage))
     return 1;

   map<string,string> extraDataMap;
   
   if(argc > 1) {
      for(int parm = 1; parm < argc; parm++) {
	 
	 if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
            cout << "Found extra data param..." << endl;
#endif
	    string td = &argv[parm][11];
	    extraDataMap = getExtraDataMap(td);
	 }
      }
   }
   
   testLogIn(test, username, password);
   
   try {
      
      epp_HostInfoReq_ref l_req(new epp_HostInfoReq());

      string trID = "ABC-12345";
      if(extraDataMap.find("trid") != extraDataMap.end()) trID = extraDataMap["trid"];
      
      l_req->m_cmd.ref(new epp_Command(NULL,
				       NULL,
				       epp_trid(trID)));
      
      if(extraDataMap.find("host") == extraDataMap.end()) {
	 l_req->m_name.ref(new epp_string("ns1.example.com"));
      }
      else {
	 l_req->m_name.ref(new epp_string(extraDataMap["host"]));
      }
      
      epp_HostInfo_ref l_obj(new epp_HostInfo());
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      epp_HostInfoRsp_ref l_rsp = l_obj->getResponseData();
      
#ifndef TEST_IO_REQUEST2CERR
      printRsp(l_rsp);
#endif
      return 0;
      
   }
   catch(const epp_XMLException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(const epp_Exception & ex) {
      cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
      cerr << "<response>" << endl;
      printResultsSeq(ex.m_details);
      printTransID(ex.m_trans_id);
      cerr << "</response>" << endl;
#endif
   }
   catch(const epptransport::epp_TrException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }
   
}
