/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/main_poll.cc,v 1.16 2003/07/18 20:21:19 tubadanm Exp $ */

#include <string>
#include <iostream>

#include "data/epp_PollReq.h"
#include "data/epp_PollRsp.h"
#include "data/epp_XMLException.h"
#include "data/epp_Exception.h"
#include "epp_Action.h"
#include "epp_Poll.h"
#include "epp_Session.h"
#include "testTools.h"
#include "data/epp_domainXMLbase.h"
#include "data/epp_contactXMLbase.h"

using namespace std;
using namespace eppobject::epp;
using namespace eppobject::domain;
using namespace eppobject::contact;

void printRsp(const epp_PollRsp_ref & req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {
      printResponsePart1(req->m_rsp);
      
      if(req->m_res_data != NULL) {
	 
	 if(req->m_res_data->getType() == "domain:trnData")
	   {
	      printResDataHeaders("domain", "trnData");
	      
	      epp_DomainTrnData_ref pdt = req->m_res_data;
	      if(pdt != NULL)
		{
		   if(pdt->m_name != NULL)
		     cerr << "\t<domain:name>" << *pdt->m_name << "</domain:name>" << endl;
		   
		   if(pdt->m_transfer_status != NULL) {
		      string trStatus = returnTransferStatusType(*pdt->m_transfer_status);
		      cerr << "\t<domain:trStatus>" << trStatus << "</domain:trStatus>" << endl;
		   }
		   
		   if(pdt->m_request_client_id != NULL)
		     cerr << "\t<domain:reID>" << *pdt->m_request_client_id << "</domain:reID>" << endl;
		   
		   if(pdt->m_request_date != NULL)
		     cerr << "\t<domain:reDate>" << *pdt->m_request_date << "</domain:reDate>" << endl;
		   
		   if(pdt->m_action_client_id != NULL)
		     cerr << "\t<domain:acID>" << *pdt->m_action_client_id << "</domain:acID>" << endl;
		   
		   if(pdt->m_action_date != NULL)
		     cerr << "\t<domain:acDate>" << *pdt->m_action_date << "</domain:acDate>" << endl;
		   
		   if(pdt->m_expiration_date != NULL)
		     cerr << "\t<domain:exDate>" << *pdt->m_expiration_date << "</domain:exDate>" << endl;
		   
		}
	      printResDataFooter("domain", "trnData");
	   }
	 else if(req->m_res_data->getType() == "contact:trnData")
	   {
	      printResDataHeaders("contact", "trnData");
	      
	      epp_ContactTrnData_ref pct = req->m_res_data;
	      if(pct != NULL)
		{
		   if(pct->m_id != NULL)
		     cerr << "\t<contact:id>" << *pct->m_id << "</contact:id>" << endl;
		   
		   if(pct->m_transfer_status != NULL) {
		      string trStatus = returnTransferStatusType(*pct->m_transfer_status);
		      cerr << "\t<contact:trStatus>" << trStatus << "</contact:trStatus>" << endl;
		   }
		   
		   if(pct->m_request_client_id != NULL)
		     cerr << "\t<contact:reID>" << *pct->m_request_client_id << "</contact:reID>" << endl;
		   
		   if(pct->m_request_date != NULL)
		     cerr << "\t<contact:reDate>" << *pct->m_request_date << "</contact:reDate>" << endl;
		   
		   if(pct->m_action_client_id != NULL)
		     cerr << "\t<contact:acID>" << *pct->m_action_client_id << "</contact:acID>" << endl;
		   
		   if(pct->m_action_date != NULL)
		     cerr << "\t<contact:acDate>" << *pct->m_action_date << "</contact:acDate>" << endl;
		   
		}
	      printResDataFooter("contact", "trnData");
	   }
	 else
	   cerr << "Unknown poll resData type" << endl;
	 
	 
      }
      printResponsePart2(req->m_rsp);
   }
   
   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {
   
   epp_Session test;

   string username, password;
   
   string extra_usage = "[-extradata=trid=xxx:op=(req|ack):id=123]";
   if(!handleArgs(test, argc, argv, username, password, extra_usage))
     return 1;
   
   map<string,string> extraDataMap;
   
   if(argc > 1) {
      for(int parm = 1; parm < argc; parm++) {
	 
	 if (!strncmp(argv[parm], "-extradata=", 11)) {
            cout << "Found extra data param..." << endl;
	    string td = &argv[parm][11];
	    extraDataMap = getExtraDataMap(td);
	 }
      }
   }
   
   testLogIn(test, username, password);
   
   try {
      // REQ-test:
      
      epp_PollReq_ref l_req(new epp_PollReq());
      
      l_req->m_cmd.ref(new epp_Command());
      
      string trID;
      if(extraDataMap.find("trid") == extraDataMap.end()) trID = "ABC-12345";
      else trID = extraDataMap["trid"];
      
      l_req->m_cmd->m_client_trid.ref(new epp_trid(trID));
      
      epp_Poll_ref l_obj(new epp_Poll());
      epp_PollRsp_ref l_rsp;
      
      if( extraDataMap.find("op") == extraDataMap.end() &&
          extraDataMap.find("id") == extraDataMap.end() ) {
	 l_req->m_op.ref(new epp_PollOpType(REQ));
	 
	 l_obj->setRequestData(*l_req);
	 
	 test.processAction(l_obj);
	 l_rsp = l_obj->getResponseData();
	 
#ifndef TEST_IO_REQUEST2CERR
	 printRsp(l_rsp);
#endif
	 
	 cout << "\n----------------------------------\n";
	 
	 // ACK-test:
	 *l_req->m_op = ACK;
	 l_req->m_msgID.ref(new epp_string("12345"));
	 l_req->m_cmd->m_client_trid.ref(new epp_trid("ABC-12346"));
      }
      else {
	 if ( extraDataMap.find("op") != extraDataMap.end() ) {
	    if(extraDataMap["op"] == "req") {
	       l_req->m_op.ref(new epp_PollOpType(REQ));
	    }
	    else if(extraDataMap["op"] == "ack") {
	       l_req->m_op.ref(new epp_PollOpType(ACK));
	    }
	 }
         if ( extraDataMap.find("id") != extraDataMap.end() ) {
            l_req->m_msgID.ref(new epp_string(extraDataMap["id"]));
	 }
      }
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      l_rsp = l_obj->getResponseData();
      
      cout << l_rsp->m_rsp << endl;
      
#ifndef TEST_IO_REQUEST2CERR
      printRsp(l_rsp);
#endif
      return 0;
      
   }
   catch(const epp_XMLException & ex) {
      cout << "epp_XMLException: " << ex.getString() << endl;
   }
   catch(const epp_Exception & ex) {
      cout << "epp_Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
      cerr << ex << endl;
      cerr << "<response>" << endl;
      printResultsSeq(ex.m_details);
      printTransID(ex.m_trans_id);
      cerr << "</response>" << endl;
#endif
   }
   catch(const epptransport::epp_TrException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }
}
